@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -356,13 +354,8 @@ photos_base_manager_dispose (GObject *object)
 
   priv = photos_base_manager_get_instance_private (self);
 
-  if (priv->objects != NULL)
-    {
-      g_hash_table_unref (priv->objects);
-      priv->objects = NULL;
-    }
-
   g_clear_object (&priv->active_object);
+  g_clear_pointer (&priv->objects, (GDestroyNotify) g_hash_table_unref);
   g_clear_pointer (&priv->sequence, (GDestroyNotify) g_sequence_free);
 
   G_OBJECT_CLASS (photos_base_manager_parent_class)->dispose (object);
@@ -622,18 +615,21 @@ photos_base_manager_get_all_filter (PhotosBaseManager *self)
   g_hash_table_iter_init (&iter, priv->objects);
   while (g_hash_table_iter_next (&iter, (gpointer *) &id, (gpointer *) &object_data))
     {
-      if (g_strcmp0 (id, "all") != 0)
-        {
-          gchar *str;
+      gchar *str;
 
-          str = photos_filterable_get_filter (PHOTOS_FILTERABLE (object_data->object));
-          if (g_strcmp0 (str, blank) == 0)
-            g_free (str);
-          else
-            {
-              strv[i] = str;
-              i++;
-            }
+      if (g_strcmp0 (id, "all") == 0)
+        continue;
+
+      if (!photos_filterable_is_search_criterion (PHOTOS_FILTERABLE (object_data->object)))
+        continue;
+
+      str = photos_filterable_get_filter (PHOTOS_FILTERABLE (object_data->object));
+      if (g_strcmp0 (str, blank) == 0)
+        g_free (str);
+      else
+        {
+          strv[i] = str;
+          i++;
         }
     }
 
