@@ -3,10 +3,10 @@
  * Copyright © 2015 Alessandro Bono
  * Copyright © 2014 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -49,16 +47,6 @@ enum
 G_DEFINE_TYPE (PhotosBaseModel, photos_base_model, G_TYPE_OBJECT);
 
 
-static void
-photos_base_model_action_state_changed (PhotosBaseModel *self, const gchar *action_name, GVariant *value)
-{
-  const gchar *id;
-
-  id = g_variant_get_string (value, NULL);
-  photos_base_manager_set_active_object_by_id (self->mngr, id);
-}
-
-
 static void
 photos_base_model_active_changed (PhotosBaseModel *self, GObject *active_object)
 {
@@ -97,17 +85,18 @@ photos_base_model_refresh (PhotosBaseModel *self)
     {
       g_autoptr (GMenuItem) menu_item = NULL;
       g_autoptr (GObject) object = NULL;
-      GVariant *target_value;
       const gchar *id;
       g_autofree gchar *name = NULL;
 
       object = g_list_model_get_object (G_LIST_MODEL (self->mngr), i);
+      if (!photos_filterable_is_search_criterion (PHOTOS_FILTERABLE (object)))
+        continue;
+
       id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
       g_object_get (object, "name", &name, NULL);
 
       menu_item = g_menu_item_new (name, NULL);
-      target_value = g_variant_new ("s", id);
-      g_menu_item_set_action_and_target_value (menu_item, action_id, target_value);
+      g_menu_item_set_action_and_target (menu_item, action_id, "s", id);
       g_menu_append_item (section, menu_item);
     }
 }
@@ -117,9 +106,6 @@ static void
 photos_base_model_constructed (GObject *object)
 {
   PhotosBaseModel *self = PHOTOS_BASE_MODEL (object);
-  GApplication *app;
-  const gchar *action_id;
-  g_autofree gchar *detailed_signal = NULL;
 
   G_OBJECT_CLASS (photos_base_model_parent_class)->constructed (object);
 
@@ -135,16 +121,6 @@ photos_base_model_constructed (GObject *object)
                            G_CALLBACK (photos_base_model_refresh),
                            self,
                            G_CONNECT_SWAPPED);
-
-  app = g_application_get_default ();
-  action_id = photos_base_manager_get_action_id (self->mngr);
-  detailed_signal = g_strconcat ("action-state-changed::", action_id, NULL);
-  g_signal_connect_object (app,
-                           detailed_signal,
-                           G_CALLBACK (photos_base_model_action_state_changed),
-                           self,
-                           G_CONNECT_SWAPPED);
-
   g_signal_connect_object (self->mngr,
                            "active-changed",
                            G_CALLBACK (photos_base_model_active_changed),
