@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2013 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -43,12 +41,14 @@ struct _PhotosCreateCollectionJob
   GObject parent_instance;
   GError *queue_error;
   PhotosTrackerQueue *queue;
+  gchar *identifier_tag;
   gchar *name;
 };
 
 enum
 {
   PROP_0,
+  PROP_IDENTIFIER_TAG,
   PROP_NAME
 };
 
@@ -124,6 +124,7 @@ photos_create_collection_job_finalize (GObject *object)
   PhotosCreateCollectionJob *self = PHOTOS_CREATE_COLLECTION_JOB (object);
 
   g_clear_error (&self->queue_error);
+  g_free (self->identifier_tag);
   g_free (self->name);
 
   G_OBJECT_CLASS (photos_create_collection_job_parent_class)->finalize (object);
@@ -137,6 +138,10 @@ photos_create_collection_job_set_property (GObject *object, guint prop_id, const
 
   switch (prop_id)
     {
+    case PROP_IDENTIFIER_TAG:
+      self->identifier_tag = g_value_dup_string (value);
+      break;
+
     case PROP_NAME:
       self->name = g_value_dup_string (value);
       break;
@@ -164,6 +169,14 @@ photos_create_collection_job_class_init (PhotosCreateCollectionJobClass *class)
   object_class->finalize = photos_create_collection_job_finalize;
   object_class->set_property = photos_create_collection_job_set_property;
 
+  g_object_class_install_property (object_class,
+                                   PROP_IDENTIFIER_TAG,
+                                   g_param_spec_string ("identifier-tag",
+                                                        "Identfier tag",
+                                                        "An optional ID that will be added to the nao:identifier",
+                                                        NULL,
+                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+
   g_object_class_install_property (object_class,
                                    PROP_NAME,
                                    g_param_spec_string ("name",
@@ -175,9 +188,12 @@ photos_create_collection_job_class_init (PhotosCreateCollectionJobClass *class)
 
 
 PhotosCreateCollectionJob *
-photos_create_collection_job_new (const gchar *name)
+photos_create_collection_job_new (const gchar *name, const gchar *identifier_tag)
 {
-  return g_object_new (PHOTOS_TYPE_CREATE_COLLECTION_JOB, "name", name, NULL);
+  g_return_val_if_fail (name != NULL && name[0] != '\0', NULL);
+  g_return_val_if_fail (identifier_tag == NULL || identifier_tag[0] != '\0', NULL);
+
+  return g_object_new (PHOTOS_TYPE_CREATE_COLLECTION_JOB, "identifier-tag", identifier_tag, "name", name, NULL);
 }
 
 
@@ -186,6 +202,7 @@ photos_create_collection_job_finish (PhotosCreateCollectionJob *self, GAsyncResu
 {
   GTask *task = G_TASK (res);
 
+  g_return_val_if_fail (PHOTOS_IS_CREATE_COLLECTION_JOB (self), NULL);
   g_return_val_if_fail (g_task_is_valid (res, self), NULL);
   g_return_val_if_fail (g_task_get_source_tag (task) == photos_create_collection_job_run, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
@@ -205,6 +222,9 @@ photos_create_collection_job_run (PhotosCreateCollectionJob *self,
   PhotosQuery *query = NULL;
   PhotosSearchContextState *state;
 
+  g_return_if_fail (PHOTOS_IS_CREATE_COLLECTION_JOB (self));
+  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_create_collection_job_run);
 
@@ -217,7 +237,7 @@ photos_create_collection_job_run (PhotosCreateCollectionJob *self,
   app = g_application_get_default ();
   state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
-  query = photos_query_builder_create_collection_query (state, self->name);
+  query = photos_query_builder_create_collection_query (state, self->name, self->identifier_tag);
   photos_tracker_queue_update_blank (self->queue,
                                      query,
                                      cancellable,
