@@ -4,10 +4,10 @@
  * Copyright © 2012 – 2017 Red Hat, Inc.
  * Copyright © 2014 Saurav Agarwalla
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -41,6 +39,7 @@
 #include "photos-selection-toolbar.h"
 #include "photos-spinner-box.h"
 #include "photos-search-context.h"
+#include "photos-search-match.h"
 #include "photos-search-type.h"
 #include "photos-search-type-manager.h"
 #include "photos-source.h"
@@ -56,6 +55,7 @@ typedef struct _PhotosEmbedSearchState PhotosEmbedSearchState;
 
 struct _PhotosEmbedSearchState
 {
+  GObject *search_match;
   GObject *search_type;
   GObject *source;
   gboolean saved;
@@ -71,6 +71,7 @@ struct _PhotosEmbed
   GtkWidget *collection_view;
   GtkWidget *collections;
   GtkWidget *favorites;
+  GtkWidget *import;
   GtkWidget *no_results;
   GtkWidget *ntfctn_mngr;
   GtkWidget *overview;
@@ -82,7 +83,8 @@ struct _PhotosEmbed
   GtkWidget *toolbar;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
-  PhotosBaseManager *srch_mngr;
+  PhotosBaseManager *srch_mtch_mngr;
+  PhotosBaseManager *srch_typ_mngr;
   PhotosEmbedSearchState search_state;
   PhotosModeController *mode_cntrlr;
   PhotosSearchController *srch_cntrlr;
@@ -102,8 +104,8 @@ static void
 photos_embed_block_search_changed (PhotosEmbed *self)
 {
   g_signal_handlers_block_by_func (self->src_mngr, photos_embed_search_changed, self);
-  g_signal_handlers_block_by_func (self->srch_mngr, photos_embed_search_changed, self);
   g_signal_handlers_block_by_func (self->srch_cntrlr, photos_embed_search_changed, self);
+  g_signal_handlers_block_by_func (self->srch_typ_mngr, photos_embed_search_changed, self);
 }
 
 
@@ -111,8 +113,8 @@ static void
 photos_embed_unblock_search_changed (PhotosEmbed *self)
 {
   g_signal_handlers_unblock_by_func (self->src_mngr, photos_embed_search_changed, self);
-  g_signal_handlers_unblock_by_func (self->srch_mngr, photos_embed_search_changed, self);
   g_signal_handlers_unblock_by_func (self->srch_cntrlr, photos_embed_search_changed, self);
+  g_signal_handlers_unblock_by_func (self->srch_typ_mngr, photos_embed_search_changed, self);
 }
 
 
@@ -155,6 +157,10 @@ photos_embed_get_view_container_from_mode (PhotosEmbed *self, PhotosWindowMode m
       view_container = self->favorites;
       break;
 
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      view_container = self->import;
+      break;
+
     case PHOTOS_WINDOW_MODE_OVERVIEW:
       view_container = self->overview;
       break;
@@ -188,18 +194,51 @@ photos_embed_activate_result (PhotosEmbed *self)
 }
 
 
+static gboolean
+photos_embed_is_search_constraint_present (PhotosEmbed *self)
+{
+  GObject *object;
+  const gchar *search_type_id;
+  const gchar *source_id;
+  const gchar *str;
+
+  object = photos_base_manager_get_active_object (self->src_mngr);
+  source_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+
+  object = photos_base_manager_get_active_object (self->srch_typ_mngr);
+  search_type_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+
+  str = photos_search_controller_get_string (self->srch_cntrlr);
+
+  return g_strcmp0 (search_type_id, PHOTOS_SEARCH_TYPE_STOCK_ALL) != 0
+    || g_strcmp0 (source_id, PHOTOS_SOURCE_STOCK_ALL) != 0
+    || (str != NULL && str [0] != '\0');
+}
+
+
 static void
 photos_embed_restore_search (PhotosEmbed *self)
 {
   GVariant *state;
+  PhotosSource *source;
+  const gchar *id;
 
   if (!self->search_state.saved)
     return;
 
   photos_embed_block_search_changed (self);
-  photos_base_manager_set_active_object (self->src_mngr, self->search_state.source);
-  photos_base_manager_set_active_object (self->srch_mngr, self->search_state.search_type);
+
+  id = photos_filterable_get_id (PHOTOS_FILTERABLE (self->search_state.source));
+  source = PHOTOS_SOURCE (photos_base_manager_get_object_by_id (self->src_mngr, id));
+  if (source == NULL)
+    id = PHOTOS_SOURCE_STOCK_ALL;
+
+  photos_base_manager_set_active_object_by_id (self->src_mngr, id);
+
+  photos_base_manager_set_active_object (self->srch_mtch_mngr, self->search_state.search_match);
+  photos_base_manager_set_active_object (self->srch_typ_mngr, self->search_state.search_type);
   photos_search_controller_set_string (self->srch_cntrlr, self->search_state.str);
+
   photos_embed_unblock_search_changed (self);
 
   self->search_state.saved = FALSE;
@@ -221,7 +260,8 @@ photos_embed_save_search (PhotosEmbed *self)
   photos_embed_clear_search (self);
 
   self->search_state.source = g_object_ref (photos_base_manager_get_active_object (self->src_mngr));
-  self->search_state.search_type = g_object_ref (photos_base_manager_get_active_object (self->srch_mngr));
+  self->search_state.search_match = g_object_ref (photos_base_manager_get_active_object (self->srch_mtch_mngr));
+  self->search_state.search_type = g_object_ref (photos_base_manager_get_active_object (self->srch_typ_mngr));
   self->search_state.str = g_strdup (photos_search_controller_get_string (self->srch_cntrlr));
   self->search_state.saved = TRUE;
 
@@ -408,6 +448,7 @@ photos_embed_prepare_for_collection_view (PhotosEmbed *self, PhotosWindowMode ol
     case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
     case PHOTOS_WINDOW_MODE_EDIT:
     case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_IMPORT:
     case PHOTOS_WINDOW_MODE_OVERVIEW:
     default:
       g_assert_not_reached ();
@@ -441,6 +482,17 @@ photos_embed_prepare_for_favorites (PhotosEmbed *self, PhotosWindowMode old_mode
 }
 
 
+static void
+photos_embed_prepare_for_import (PhotosEmbed *self, PhotosWindowMode old_mode)
+{
+  if (old_mode == PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_embed_tracker_controllers_set_frozen (self, FALSE);
+
+  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (self->spinner_box));
+  gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "import");
+}
+
+
 static void
 photos_embed_prepare_for_overview (PhotosEmbed *self, PhotosWindowMode old_mode)
 {
@@ -460,8 +512,15 @@ photos_embed_prepare_for_search (PhotosEmbed *self, PhotosWindowMode old_mode)
   if (old_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     photos_embed_tracker_controllers_set_frozen (self, FALSE);
 
-  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (self->spinner_box));
-  gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "search");
+  if (photos_embed_is_search_constraint_present (self))
+    {
+      photos_spinner_box_stop (PHOTOS_SPINNER_BOX (self->spinner_box));
+      gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "search");
+    }
+  else
+    {
+      photos_mode_controller_go_back (self->mode_cntrlr);
+    }
 }
 
 
@@ -478,53 +537,83 @@ photos_embed_query_status_changed (PhotosEmbed *self, gboolean querying)
 static void
 photos_embed_search_changed (PhotosEmbed *self)
 {
-  GObject *object;
-  const gchar *search_type_id;
-  const gchar *source_id;
-  const gchar *str;
+  GMount *mount;
+  PhotosSource *source;
 
-  /* Whenever a search constraint is specified, we switch to the
-   * search mode. Search is always global. If we are in
-   * collection-view, we go back to the previous top-level mode and
-   * then enter the search mode.
-   *
-   * When all constraints have been lifted we want to go back to the
-   * previous top-level mode which can be either collections,
-   * favorites or overview.
-   *
-   * The constraints are saved when entering collection-view or
-   * preview from the search mode. They are restored when going back.
-   * Saving and restoring doesn't cause any further mode changes.
-   */
+  source = PHOTOS_SOURCE (photos_base_manager_get_active_object (self->src_mngr));
 
-  self->search_changed = TRUE;
-
-  object = photos_base_manager_get_active_object (self->src_mngr);
-  source_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
-
-  object = photos_base_manager_get_active_object (self->srch_mngr);
-  search_type_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
-
-  str = photos_search_controller_get_string (self->srch_cntrlr);
-
-  if (g_strcmp0 (search_type_id, PHOTOS_SEARCH_TYPE_STOCK_ALL) == 0
-      && g_strcmp0 (source_id, PHOTOS_SOURCE_STOCK_ALL) == 0
-      && (str == NULL || str [0] == '\0'))
-    {
-      photos_mode_controller_go_back (self->mode_cntrlr);
-    }
-  else
+  mount = photos_source_get_mount (source);
+  if (mount != NULL)
     {
+      GObject *object;
       PhotosWindowMode mode;
+      const gchar *search_match_id;
+      const gchar *search_type_id;
 
       mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+      g_return_if_fail (mode != PHOTOS_WINDOW_MODE_NONE);
+      g_return_if_fail (mode != PHOTOS_WINDOW_MODE_EDIT);
+      g_return_if_fail (mode != PHOTOS_WINDOW_MODE_IMPORT);
+      g_return_if_fail (mode != PHOTOS_WINDOW_MODE_PREVIEW);
+
+      photos_embed_block_search_changed (self);
+
       if (mode == PHOTOS_WINDOW_MODE_COLLECTION_VIEW)
         photos_mode_controller_go_back (self->mode_cntrlr);
 
-      photos_mode_controller_set_window_mode (self->mode_cntrlr, PHOTOS_WINDOW_MODE_SEARCH);
-    }
+      mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+      if (mode == PHOTOS_WINDOW_MODE_SEARCH)
+        photos_mode_controller_go_back (self->mode_cntrlr);
 
-  self->search_changed = FALSE;
+      photos_base_manager_set_active_object (self->src_mngr, G_OBJECT (source));
+
+      object = photos_base_manager_get_active_object (self->srch_mtch_mngr);
+      search_match_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+      g_assert_cmpstr (search_match_id, ==, PHOTOS_SEARCH_MATCH_STOCK_ALL);
+
+      object = photos_base_manager_get_active_object (self->srch_typ_mngr);
+      search_type_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+      g_assert_cmpstr (search_type_id, ==, PHOTOS_SEARCH_TYPE_STOCK_ALL);
+
+      photos_mode_controller_set_window_mode (self->mode_cntrlr, PHOTOS_WINDOW_MODE_IMPORT);
+
+      photos_embed_unblock_search_changed (self);
+    }
+  else
+    {
+      /* Whenever a search constraint is specified, we switch to the
+       * search mode. Search is always global. If we are in
+       * collection-view, we go back to the previous top-level mode and
+       * then enter the search mode.
+       *
+       * When all constraints have been lifted we want to go back to the
+       * previous top-level mode which can be either collections,
+       * favorites or overview.
+       *
+       * The constraints are saved when entering collection-view or
+       * preview from the search mode. They are restored when going back.
+       * Saving and restoring doesn't cause any further mode changes.
+       */
+
+      self->search_changed = TRUE;
+
+      if (photos_embed_is_search_constraint_present (self))
+        {
+          PhotosWindowMode mode;
+
+          mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+          if (mode == PHOTOS_WINDOW_MODE_COLLECTION_VIEW)
+            photos_mode_controller_go_back (self->mode_cntrlr);
+
+          photos_mode_controller_set_window_mode (self->mode_cntrlr, PHOTOS_WINDOW_MODE_SEARCH);
+        }
+      else
+        {
+          photos_mode_controller_go_back (self->mode_cntrlr);
+        }
+
+      self->search_changed = FALSE;
+    }
 }
 
 
@@ -619,6 +708,7 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
 
     case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
     case PHOTOS_WINDOW_MODE_EDIT:
+    case PHOTOS_WINDOW_MODE_IMPORT:
     case PHOTOS_WINDOW_MODE_PREVIEW:
     case PHOTOS_WINDOW_MODE_SEARCH:
       break;
@@ -648,6 +738,10 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
       photos_embed_prepare_for_favorites (self, old_mode);
       break;
 
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      photos_embed_prepare_for_import (self, old_mode);
+      break;
+
     case PHOTOS_WINDOW_MODE_OVERVIEW:
       photos_embed_prepare_for_overview (self, old_mode);
       break;
@@ -681,7 +775,8 @@ photos_embed_dispose (GObject *object)
   g_clear_object (&self->ntfctn_mngr);
   g_clear_object (&self->item_mngr);
   g_clear_object (&self->src_mngr);
-  g_clear_object (&self->srch_mngr);
+  g_clear_object (&self->srch_mtch_mngr);
+  g_clear_object (&self->srch_typ_mngr);
   g_clear_object (&self->mode_cntrlr);
   g_clear_object (&self->srch_cntrlr);
   g_clear_object (&self->trk_ovrvw_cntrlr);
@@ -736,6 +831,9 @@ photos_embed_init (PhotosEmbed *self)
   name = photos_view_container_get_name (PHOTOS_VIEW_CONTAINER (self->favorites));
   gtk_stack_add_titled (GTK_STACK (self->stack), self->favorites, "favorites", name);
 
+  self->import = photos_view_container_new (PHOTOS_WINDOW_MODE_IMPORT, _("Import"));
+  gtk_stack_add_named (GTK_STACK (self->stack), self->import, "import");
+
   self->search = photos_view_container_new (PHOTOS_WINDOW_MODE_SEARCH, _("Search"));
   gtk_stack_add_named (GTK_STACK (self->stack), self->search, "search");
 
@@ -799,8 +897,10 @@ photos_embed_init (PhotosEmbed *self)
                            self,
                            G_CONNECT_SWAPPED);
 
-  self->srch_mngr = g_object_ref (state->srch_typ_mngr);
-  g_signal_connect_object (self->srch_mngr,
+  self->srch_mtch_mngr = g_object_ref (state->srch_mtch_mngr);
+
+  self->srch_typ_mngr = g_object_ref (state->srch_typ_mngr);
+  g_signal_connect_object (self->srch_typ_mngr,
                            "active-changed",
                            G_CALLBACK (photos_embed_search_changed),
                            self,
