@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -56,10 +54,10 @@ photos_filterable_get_builtin (PhotosFilterable *self)
 
 
 gchar *
-photos_filterable_get_filter (PhotosFilterable *iface)
+photos_filterable_get_filter (PhotosFilterable *self)
 {
-  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (iface), NULL);
-  return PHOTOS_FILTERABLE_GET_IFACE (iface)->get_filter (iface);
+  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (self), NULL);
+  return PHOTOS_FILTERABLE_GET_IFACE (self)->get_filter (self);
 }
 
 
@@ -72,8 +70,16 @@ photos_filterable_get_id (PhotosFilterable *self)
 
 
 gchar *
-photos_filterable_get_where (PhotosFilterable *iface)
+photos_filterable_get_where (PhotosFilterable *self)
 {
-  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (iface), NULL);
-  return PHOTOS_FILTERABLE_GET_IFACE (iface)->get_where (iface);
+  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (self), NULL);
+  return PHOTOS_FILTERABLE_GET_IFACE (self)->get_where (self);
+}
+
+
+gboolean
+photos_filterable_is_search_criterion (PhotosFilterable *self)
+{
+  g_return_val_if_fail (PHOTOS_IS_FILTERABLE (self), FALSE);
+  return PHOTOS_FILTERABLE_GET_IFACE (self)->is_search_criterion (self);
 }
