@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2013 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -35,8 +33,6 @@
 #include "photos-operation-jpg-guess-sizes.h"
 #include "photos-operation-png-guess-sizes.h"
 #include "photos-operation-saturation.h"
-#include "photos-operation-shadows-highlights.h"
-#include "photos-operation-shadows-highlights-correction.h"
 #include "photos-operation-svg-multiply.h"
 #include "photos-quarks.h"
 
@@ -55,16 +51,15 @@ static const gchar *REQUIRED_GEGL_OPS[] =
 {
   "gegl:buffer-sink",
   "gegl:buffer-source",
-  "gegl:convert-format",
   "gegl:crop",
   "gegl:exposure",
-  "gegl:gaussian-blur",
   "gegl:gray",
   "gegl:load",
   "gegl:noise-reduction",
   "gegl:nop",
   "gegl:save-pixbuf",
   "gegl:scale-ratio",
+  "gegl:shadows-highlights",
   "gegl:unsharp-mask",
 
   /* Used by gegl:load */
@@ -568,8 +563,6 @@ photos_gegl_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_OPERATION_JPG_GUESS_SIZES);
       g_type_ensure (PHOTOS_TYPE_OPERATION_PNG_GUESS_SIZES);
       g_type_ensure (PHOTOS_TYPE_OPERATION_SATURATION);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_SHADOWS_HIGHLIGHTS);
-      g_type_ensure (PHOTOS_TYPE_OPERATION_SHADOWS_HIGHLIGHTS_CORRECTION);
       g_type_ensure (PHOTOS_TYPE_OPERATION_SVG_MULTIPLY);
 
       g_once_init_leave (&once_init_value, 1);
