@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -32,6 +30,8 @@
 #include <tracker-sparql.h>
 
 #include "egg-counter.h"
+#include "photos-debug.h"
+#include "photos-device-item.h"
 #include "photos-enums.h"
 #include "photos-filterable.h"
 #include "photos-item-manager.h"
@@ -42,6 +42,7 @@
 #include "photos-single-item-job.h"
 #include "photos-tracker-change-event.h"
 #include "photos-tracker-change-monitor.h"
+#include "photos-tracker-queue.h"
 #include "photos-utils.h"
 
 
@@ -59,9 +60,11 @@ struct _PhotosItemManager
   PhotosBaseManager **item_mngr_chldrn;
   PhotosLoadState load_state;
   PhotosTrackerChangeMonitor *monitor;
+  PhotosTrackerQueue *queue;
   PhotosWindowMode mode;
   gboolean fullscreen;
   gboolean *constrain_additions;
+  guint wait_for_changes_id;
 };
 
 enum
@@ -95,10 +98,19 @@ struct _PhotosItemManagerHiddenItem
 };
 
 
+enum
+{
+  WAIT_FOR_CHANGES_TIMEOUT = 1 /* s */
+};
+
+
+static gboolean photos_item_manager_wait_for_changes_timeout (gpointer user_data);
+
+
 static PhotosItemManagerHiddenItem *
 photos_item_manager_hidden_item_new (PhotosBaseItem *item)
 {
-  GEnumClass *window_mode_class;
+  GEnumClass *window_mode_class; /* TODO: use g_autoptr */
   PhotosItemManagerHiddenItem *hidden_item;
 
   hidden_item = g_slice_new0 (PhotosItemManagerHiddenItem);
@@ -132,7 +144,7 @@ photos_item_manager_add_object (PhotosBaseManager *mngr, GObject *object)
 static gboolean
 photos_item_manager_can_add_mtime_for_mode (PhotosItemManager *self, gint64 mtime, PhotosWindowMode mode)
 {
-  PhotosBaseItem *oldest_item = NULL;
+  g_autoptr (PhotosBaseItem) oldest_item = NULL;
   PhotosBaseManager *item_mngr_chld;
   gboolean ret_val = TRUE;
   gint64 oldest_mtime;
@@ -155,7 +167,6 @@ photos_item_manager_can_add_mtime_for_mode (PhotosItemManager *self, gint64 mtim
   ret_val = FALSE;
 
  out:
-  g_clear_object (&oldest_item);
   return ret_val;
 }
 
@@ -189,11 +200,16 @@ photos_item_manager_can_add_item_for_mode (PhotosItemManager *self, PhotosBaseIt
 static gboolean
 photos_item_manager_cursor_is_collection (TrackerSparqlCursor *cursor)
 {
-  gboolean ret_val;
+  gboolean ret_val = FALSE;
   const gchar *rdf_type;
 
   rdf_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_RDF_TYPE, NULL);
+  if (rdf_type == NULL)
+    goto out;
+
   ret_val = strstr (rdf_type, "nfo#DataContainer") != NULL;
+
+ out:
   return ret_val;
 }
 
@@ -226,6 +242,7 @@ static void
 photos_item_manager_info_updated (PhotosBaseItem *item, gpointer user_data)
 {
   PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
+  GType base_item_type;
   PhotosBaseItem *updated_item;
   gboolean is_collection;
   gboolean is_favorite;
@@ -235,11 +252,12 @@ photos_item_manager_info_updated (PhotosBaseItem *item, gpointer user_data)
 
   id = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   updated_item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (PHOTOS_BASE_MANAGER (self), id));
-  if (updated_item == NULL)
-    return;
-
   g_return_if_fail (updated_item == item);
 
+  base_item_type = G_OBJECT_TYPE (item);
+  if (base_item_type == PHOTOS_TYPE_DEVICE_ITEM)
+    goto out;
+
   is_collection = photos_base_item_is_collection (item);
   is_favorite = photos_base_item_is_favorite (item);
 
@@ -269,21 +287,28 @@ photos_item_manager_info_updated (PhotosBaseItem *item, gpointer user_data)
       if (!is_favorite)
         photos_base_manager_remove_object (self->item_mngr_chldrn[PHOTOS_WINDOW_MODE_FAVORITES], G_OBJECT (item));
     }
+
+ out:
+  return;
 }
 
 
 static void
 photos_item_manager_add_cursor_for_mode (PhotosItemManager *self,
+                                         GType base_item_type,
                                          TrackerSparqlCursor *cursor,
                                          PhotosWindowMode mode,
                                          gboolean force)
 {
-  PhotosBaseItem *item = NULL;
+  g_autoptr (PhotosBaseItem) item = NULL;
   PhotosBaseManager *item_mngr_chld;
   gboolean is_collection;
   const gchar *id;
 
   g_return_if_fail (PHOTOS_IS_ITEM_MANAGER (self));
+  g_return_if_fail (base_item_type == G_TYPE_NONE
+                    || (base_item_type != PHOTOS_TYPE_BASE_ITEM
+                        && g_type_is_a (base_item_type, PHOTOS_TYPE_BASE_ITEM)));
   g_return_if_fail (TRACKER_SPARQL_IS_CURSOR (cursor));
   g_return_if_fail (mode != PHOTOS_WINDOW_MODE_NONE);
   g_return_if_fail (mode != PHOTOS_WINDOW_MODE_EDIT);
@@ -317,7 +342,7 @@ photos_item_manager_add_cursor_for_mode (PhotosItemManager *self,
         }
       else
         {
-          item = photos_item_manager_create_item (self, cursor);
+          item = photos_item_manager_create_item (self, base_item_type, cursor, TRUE);
           if (photos_base_item_is_collection (item))
             g_hash_table_insert (self->collections, g_strdup (id), g_object_ref (item));
 
@@ -332,7 +357,7 @@ photos_item_manager_add_cursor_for_mode (PhotosItemManager *self,
     }
 
  out:
-  g_clear_object (&item);
+  return;
 }
 
 
@@ -345,6 +370,8 @@ photos_item_manager_check_wait_for_changes (PhotosItemManager *self, const gchar
   g_return_if_fail (id != NULL && id[0] != '\0');
   g_return_if_fail (uri != NULL && uri[0] != '\0');
 
+  photos_debug (PHOTOS_DEBUG_TRACKER, "Detected changes to %s", uri);
+
   tasks = (GList *) g_hash_table_lookup (self->wait_for_changes_table, uri);
   for (l = tasks; l != NULL; l = l->next)
     {
@@ -357,25 +384,83 @@ photos_item_manager_check_wait_for_changes (PhotosItemManager *self, const gchar
 
 
 static void
-photos_item_manager_item_created_executed (GObject *source_object, GAsyncResult *res, gpointer user_data)
+photos_item_manager_item_created_executed_import (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
-  GError *error = NULL;
-  PhotosSingleItemJob *job = PHOTOS_SINGLE_ITEM_JOB (source_object);
-  TrackerSparqlCursor *cursor = NULL;
+  g_autoptr (PhotosItemManager) self = PHOTOS_ITEM_MANAGER (user_data);
+  PhotosSingleItemJob *job_import = PHOTOS_SINGLE_ITEM_JOB (source_object);
+  TrackerSparqlCursor *cursor = NULL; /* TODO: use g_autoptr */
 
-  cursor = photos_single_item_job_finish (job, res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to query single item: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
+  {
+    g_autoptr (GError) error = NULL;
+
+    cursor = photos_single_item_job_finish (job_import, res, &error);
+    if (error != NULL)
+      {
+        g_warning ("Unable to query single item: %s", error->message);
+        goto out;
+      }
+  }
 
   if (cursor == NULL)
     goto out;
 
-  photos_item_manager_add_item (self, cursor, FALSE);
+  photos_item_manager_add_item_for_mode (self, PHOTOS_TYPE_DEVICE_ITEM, PHOTOS_WINDOW_MODE_IMPORT, cursor);
+
+ out:
+  g_clear_object (&cursor);
+}
+
+
+static void
+photos_item_manager_item_created_executed_overview (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  g_autoptr (PhotosItemManager) self = PHOTOS_ITEM_MANAGER (user_data);
+  PhotosSingleItemJob *job_overview = PHOTOS_SINGLE_ITEM_JOB (source_object);
+  TrackerSparqlCursor *cursor = NULL; /* TODO: use g_autoptr */
+
+  {
+    g_autoptr (GError) error = NULL;
+
+    cursor = photos_single_item_job_finish (job_overview, res, &error);
+    if (error != NULL)
+      {
+        g_warning ("Unable to query single item: %s", error->message);
+        goto out;
+      }
+  }
+
+  if (cursor == NULL)
+    goto out;
+
+  photos_item_manager_add_item (self, G_TYPE_NONE, cursor, FALSE);
+
+ out:
+  g_clear_object (&cursor);
+}
+
+
+static void
+photos_item_manager_item_created_executed_wait_for_changes (GObject *source_object,
+                                                            GAsyncResult *res,
+                                                            gpointer user_data)
+{
+  g_autoptr (PhotosItemManager) self = PHOTOS_ITEM_MANAGER (user_data);
+  PhotosSingleItemJob *job_wait_for_changes = PHOTOS_SINGLE_ITEM_JOB (source_object);
+  TrackerSparqlCursor *cursor = NULL; /* TODO: use g_autoptr */
+
+  {
+    g_autoptr (GError) error = NULL;
+
+    cursor = photos_single_item_job_finish (job_wait_for_changes, res, &error);
+    if (error != NULL)
+      {
+        g_warning ("Unable to query single item: %s", error->message);
+        goto out;
+      }
+  }
+
+  if (cursor == NULL)
+    goto out;
 
   if (!photos_item_manager_cursor_is_collection (cursor))
     {
@@ -384,12 +469,12 @@ photos_item_manager_item_created_executed (GObject *source_object, GAsyncResult
 
       id = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URN, NULL);
       uri = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URI, NULL);
-      photos_item_manager_check_wait_for_changes (self, id, uri);
+      if (id != NULL && id[0] != '\0' && uri != NULL && uri[0] != '\0')
+        photos_item_manager_check_wait_for_changes (self, id, uri);
     }
 
  out:
   g_clear_object (&cursor);
-  g_object_unref (self);
 }
 
 
@@ -397,24 +482,47 @@ static void
 photos_item_manager_item_created (PhotosItemManager *self, const gchar *urn)
 {
   GApplication *app;
-  PhotosBaseItem *old_hidden_item;
+  PhotosItemManagerHiddenItem *old_hidden_item;
   PhotosSearchContextState *state;
-  PhotosSingleItemJob *job;
+  g_autoptr (PhotosSingleItemJob) job_import = NULL;
+  g_autoptr (PhotosSingleItemJob) job_overview = NULL;
+  guint wait_for_changes_size;
 
-  old_hidden_item = PHOTOS_BASE_ITEM (g_hash_table_lookup (self->hidden_items, urn));
+  old_hidden_item = (PhotosItemManagerHiddenItem *) g_hash_table_lookup (self->hidden_items, urn);
   g_return_if_fail (old_hidden_item == NULL);
 
   app = g_application_get_default ();
   state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
-  job = photos_single_item_job_new (urn);
-  photos_single_item_job_run (job,
+  job_import = photos_single_item_job_new (urn);
+  photos_single_item_job_run (job_import,
+                              state,
+                              PHOTOS_QUERY_FLAGS_IMPORT,
+                              NULL,
+                              photos_item_manager_item_created_executed_import,
+                              g_object_ref (self));
+
+  job_overview = photos_single_item_job_new (urn);
+  photos_single_item_job_run (job_overview,
                               state,
                               PHOTOS_QUERY_FLAGS_NONE,
                               NULL,
-                              photos_item_manager_item_created_executed,
+                              photos_item_manager_item_created_executed_overview,
                               g_object_ref (self));
-  g_object_unref (job);
+
+  wait_for_changes_size = g_hash_table_size (self->wait_for_changes_table);
+  if (wait_for_changes_size > 0)
+    {
+      g_autoptr (PhotosSingleItemJob) job_wait_for_changes = NULL;
+
+      job_wait_for_changes = photos_single_item_job_new (urn);
+      photos_single_item_job_run (job_wait_for_changes,
+                                  state,
+                                  PHOTOS_QUERY_FLAGS_UNFILTERED,
+                                  NULL,
+                                  photos_item_manager_item_created_executed_wait_for_changes,
+                                  g_object_ref (self));
+    }
 }
 
 
@@ -499,6 +607,130 @@ photos_item_manager_cursor_is_favorite (TrackerSparqlCursor *cursor)
 }
 
 
+static void
+photos_item_manager_remove_timeout (PhotosItemManager *self)
+{
+  if (self->wait_for_changes_id != 0)
+    {
+      g_source_remove (self->wait_for_changes_id);
+      self->wait_for_changes_id = 0;
+    }
+}
+
+
+static void
+photos_item_manager_wait_for_changes_timeout_cursor_next (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
+  TrackerSparqlCursor *cursor = TRACKER_SPARQL_CURSOR (source_object);
+  gboolean success;
+  const gchar *id;
+  const gchar *uri;
+  guint wait_for_changes_size;
+
+  {
+    g_autoptr (GError) error = NULL;
+
+    /* Note that tracker_sparql_cursor_next_finish can return FALSE even
+     * without an error.
+     */
+    success = tracker_sparql_cursor_next_finish (cursor, res, &error);
+    if (error != NULL)
+      {
+        g_warning ("Unable to fetch URN for URI: %s", error->message);
+        goto out;
+      }
+  }
+
+  if (!success)
+    goto out;
+
+  id = tracker_sparql_cursor_get_string (cursor, 0, NULL);
+  uri = tracker_sparql_cursor_get_string (cursor, 1, NULL);
+  if (id != NULL && id[0] != '\0' && uri != NULL && uri[0] != '\0')
+    photos_item_manager_check_wait_for_changes (self, id, uri);
+
+  photos_item_manager_remove_timeout (self);
+
+  wait_for_changes_size = g_hash_table_size (self->wait_for_changes_table);
+  if (wait_for_changes_size > 0)
+    {
+      self->wait_for_changes_id = g_timeout_add_seconds (WAIT_FOR_CHANGES_TIMEOUT,
+                                                         photos_item_manager_wait_for_changes_timeout,
+                                                         self);
+    }
+
+ out:
+  g_object_unref (self);
+}
+
+
+static void
+photos_item_manager_wait_for_changes_timeout_query_executed (GObject *source_object,
+                                                             GAsyncResult *res,
+                                                             gpointer user_data)
+{
+  PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
+  TrackerSparqlConnection *connection = TRACKER_SPARQL_CONNECTION (source_object);
+  TrackerSparqlCursor *cursor = NULL; /* TODO: use g_autoptr */
+
+  {
+    g_autoptr (GError) error = NULL;
+
+    cursor = tracker_sparql_connection_query_finish (connection, res, &error);
+    if (error != NULL)
+      {
+        g_warning ("Unable to fetch URN for URI: %s", error->message);
+        goto out;
+      }
+  }
+
+  if (cursor == NULL)
+    goto out;
+
+  tracker_sparql_cursor_next_async (cursor,
+                                    NULL,
+                                    photos_item_manager_wait_for_changes_timeout_cursor_next,
+                                    g_object_ref (self));
+
+ out:
+  g_clear_object (&cursor);
+}
+
+
+static gboolean
+photos_item_manager_wait_for_changes_timeout (gpointer user_data)
+{
+  PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
+  GHashTableIter iter;
+  GList *tasks;
+  const gchar *uri;
+
+  if (G_UNLIKELY (self->queue == NULL))
+    goto out;
+
+  g_hash_table_iter_init (&iter, self->wait_for_changes_table);
+  while (g_hash_table_iter_next (&iter, (gpointer *) &uri, (gpointer *) &tasks))
+    {
+      g_autoptr (PhotosQuery) query = NULL;
+      g_autofree gchar *sparql = NULL;
+
+      sparql = g_strdup_printf ("SELECT ?urn nie:url (?urn) WHERE { ?urn nie:url '%s' }", uri);
+      query = photos_query_new (NULL, sparql);
+      photos_tracker_queue_select (self->queue,
+                                   query,
+                                   NULL,
+                                   photos_item_manager_wait_for_changes_timeout_query_executed,
+                                   g_object_ref (self),
+                                   g_object_unref);
+    }
+
+ out:
+  self->wait_for_changes_id = 0;
+  return G_SOURCE_REMOVE;
+}
+
+
 static GObject *
 photos_item_manager_get_active_object (PhotosBaseManager *mngr)
 {
@@ -552,7 +784,7 @@ photos_item_manager_get_where (PhotosBaseManager *mngr, gint flags)
 {
   PhotosItemManager *self = PHOTOS_ITEM_MANAGER (mngr);
 
-  if (self->active_collection == NULL)
+  if (self->active_collection == NULL || (flags & PHOTOS_QUERY_FLAGS_SEARCH) != 0)
     return g_strdup ("");
 
   return photos_base_item_get_where (self->active_collection);
@@ -562,32 +794,30 @@ photos_item_manager_get_where (PhotosBaseManager *mngr, gint flags)
 static void
 photos_item_manager_item_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  PhotosItemManager *self = PHOTOS_ITEM_MANAGER (user_data);
-  GError *error;
-  GeglNode *node = NULL;
+  g_autoptr (PhotosItemManager) self = PHOTOS_ITEM_MANAGER (user_data);
+  g_autoptr (GeglNode) node = NULL;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
 
   g_clear_object (&self->loader_cancellable);
 
-  error = NULL;
-  node = photos_base_item_load_finish (item, res, &error);
-  if (error != NULL)
-    {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_warning ("Unable to load the item: %s", error->message);
+  {
+    g_autoptr (GError) error = NULL;
 
-      self->load_state = PHOTOS_LOAD_STATE_ERROR;
-      g_error_free (error);
-    }
-  else
-    {
-      self->load_state = PHOTOS_LOAD_STATE_FINISHED;
-    }
+    node = photos_base_item_load_finish (item, res, &error);
+    if (error != NULL)
+      {
+        if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+          g_warning ("Unable to load the item: %s", error->message);
+
+        self->load_state = PHOTOS_LOAD_STATE_ERROR;
+      }
+    else
+      {
+        self->load_state = PHOTOS_LOAD_STATE_FINISHED;
+      }
+  }
 
   g_signal_emit (self, signals[LOAD_FINISHED], 0, item, node);
-
-  g_clear_object (&node);
-  g_object_unref (self);
 }
 
 
@@ -602,7 +832,7 @@ static void
 photos_item_manager_remove_object_by_id (PhotosBaseManager *mngr, const gchar *id)
 {
   PhotosItemManager *self = PHOTOS_ITEM_MANAGER (mngr);
-  PhotosBaseItem *item;
+  g_autoptr (PhotosBaseItem) item = NULL;
   guint i;
 
   g_hash_table_remove (self->collections, id);
@@ -618,7 +848,6 @@ photos_item_manager_remove_object_by_id (PhotosBaseManager *mngr, const gchar *i
     photos_base_manager_remove_object_by_id (self->item_mngr_chldrn[i], id);
 
   g_signal_emit_by_name (self, "object-removed", G_OBJECT (item));
-  g_object_unref (item);
 }
 
 
@@ -673,6 +902,8 @@ photos_item_manager_set_active_object (PhotosBaseManager *manager, GObject *obje
 
   g_return_val_if_fail (object != NULL, FALSE);
   g_return_val_if_fail (PHOTOS_IS_BASE_ITEM (object), FALSE);
+  g_return_val_if_fail (self->mode != PHOTOS_WINDOW_MODE_EDIT, FALSE);
+  g_return_val_if_fail (self->mode != PHOTOS_WINDOW_MODE_IMPORT, FALSE);
 
   is_collection = photos_base_item_is_collection (PHOTOS_BASE_ITEM (object));
   if (is_collection)
@@ -772,6 +1003,8 @@ photos_item_manager_dispose (GObject *object)
 {
   PhotosItemManager *self = PHOTOS_ITEM_MANAGER (object);
 
+  photos_item_manager_remove_timeout (self);
+
   if (self->item_mngr_chldrn != NULL)
     {
       guint i;
@@ -790,6 +1023,7 @@ photos_item_manager_dispose (GObject *object)
   g_clear_object (&self->loader_cancellable);
   g_clear_object (&self->active_collection);
   g_clear_object (&self->monitor);
+  g_clear_object (&self->queue);
 
   G_OBJECT_CLASS (photos_item_manager_parent_class)->dispose (object);
 }
@@ -812,7 +1046,7 @@ photos_item_manager_finalize (GObject *object)
 static void
 photos_item_manager_init (PhotosItemManager *self)
 {
-  GEnumClass *window_mode_class;
+  GEnumClass *window_mode_class; /* TODO: use g_autoptr */
   guint i;
 
   EGG_COUNTER_INC (instances);
@@ -851,6 +1085,14 @@ photos_item_manager_init (PhotosItemManager *self)
                              self,
                              G_CONNECT_SWAPPED);
 
+  {
+    g_autoptr (GError) error = NULL;
+
+    self->queue = photos_tracker_queue_dup_singleton (NULL, &error);
+    if (G_UNLIKELY (error != NULL))
+      g_warning ("Unable to create PhotosTrackerQueue: %s", error->message);
+  }
+
   self->fullscreen = FALSE;
   self->constrain_additions = (gboolean *) g_malloc0_n (window_mode_class->n_values, sizeof (gboolean));
 
@@ -959,33 +1201,82 @@ photos_item_manager_new (void)
 
 
 void
-photos_item_manager_add_item (PhotosItemManager *self, TrackerSparqlCursor *cursor, gboolean force)
+photos_item_manager_add_item (PhotosItemManager *self,
+                              GType base_item_type,
+                              TrackerSparqlCursor *cursor,
+                              gboolean force)
 {
+  PhotosItemManagerHiddenItem *old_hidden_item;
+  const gchar *id;
+
   g_return_if_fail (PHOTOS_IS_ITEM_MANAGER (self));
+  g_return_if_fail (base_item_type == G_TYPE_NONE
+                    || (base_item_type != PHOTOS_TYPE_BASE_ITEM
+                        && g_type_is_a (base_item_type, PHOTOS_TYPE_BASE_ITEM)));
   g_return_if_fail (TRACKER_SPARQL_IS_CURSOR (cursor));
 
+  id = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URN, NULL);
+  g_return_if_fail (id != NULL && id[0] != '\0');
+
+  old_hidden_item = (PhotosItemManagerHiddenItem *) g_hash_table_lookup (self->hidden_items, id);
+  if (old_hidden_item != NULL)
+    goto out;
+
   if (photos_item_manager_cursor_is_collection (cursor))
     {
-      if (self->active_collection != NULL && force)
-        photos_mode_controller_go_back (self);
-
-      if (self->active_collection == NULL)
-        photos_item_manager_add_cursor_for_mode (self, cursor, PHOTOS_WINDOW_MODE_COLLECTIONS, force);
+      photos_item_manager_add_cursor_for_mode (self,
+                                               base_item_type,
+                                               cursor,
+                                               PHOTOS_WINDOW_MODE_COLLECTIONS,
+                                               force);
     }
   else
     {
       if (photos_item_manager_cursor_is_favorite (cursor))
-        photos_item_manager_add_cursor_for_mode (self, cursor, PHOTOS_WINDOW_MODE_FAVORITES, force);
+        {
+          photos_item_manager_add_cursor_for_mode (self,
+                                                   base_item_type,
+                                                   cursor,
+                                                   PHOTOS_WINDOW_MODE_FAVORITES,
+                                                   force);
+        }
 
-      photos_item_manager_add_cursor_for_mode (self, cursor, PHOTOS_WINDOW_MODE_OVERVIEW, force);
+      photos_item_manager_add_cursor_for_mode (self,
+                                               base_item_type,
+                                               cursor,
+                                               PHOTOS_WINDOW_MODE_OVERVIEW,
+                                               force);
     }
+
+ out:
+  return;
 }
 
 
 void
-photos_item_manager_add_item_for_mode (PhotosItemManager *self, PhotosWindowMode mode, TrackerSparqlCursor *cursor)
+photos_item_manager_add_item_for_mode (PhotosItemManager *self,
+                                       GType base_item_type,
+                                       PhotosWindowMode mode,
+                                       TrackerSparqlCursor *cursor)
 {
-  photos_item_manager_add_cursor_for_mode (self, cursor, mode, FALSE);
+  PhotosItemManagerHiddenItem *old_hidden_item;
+  const gchar *id;
+
+  g_return_if_fail (base_item_type == G_TYPE_NONE
+                    || (base_item_type != PHOTOS_TYPE_BASE_ITEM
+                        && g_type_is_a (base_item_type, PHOTOS_TYPE_BASE_ITEM)));
+
+  id = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URN, NULL);
+  g_return_if_fail (id != NULL && id[0] != '\0');
+
+  old_hidden_item = (PhotosItemManagerHiddenItem *) g_hash_table_lookup (self->hidden_items, id);
+  if (old_hidden_item != NULL)
+    goto out;
+
+  photos_item_manager_add_cursor_for_mode (self, base_item_type, cursor, mode, FALSE);
+
+ out:
+  return;
 }
 
 
@@ -1005,7 +1296,7 @@ photos_item_manager_clear (PhotosItemManager *self, PhotosWindowMode mode)
   n_items = g_list_model_get_n_items (G_LIST_MODEL (item_mngr_chld));
   for (i = 0; i < n_items; i++)
     {
-      PhotosBaseItem *item;
+      g_autoptr (PhotosBaseItem) item = NULL;
       PhotosBaseItem *item1 = NULL;
       const gchar *id;
       guint j;
@@ -1028,10 +1319,9 @@ photos_item_manager_clear (PhotosItemManager *self, PhotosWindowMode mode)
           item1 = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (self->item_mngr_chldrn[0], id));
           g_assert_true (item == item1);
 
+          g_signal_handlers_disconnect_by_func (item, photos_item_manager_info_updated, self);
           photos_base_manager_remove_object_by_id (self->item_mngr_chldrn[0], id);
         }
-
-      g_object_unref (item);
     }
 
   photos_base_manager_clear (item_mngr_chld);
@@ -1039,53 +1329,82 @@ photos_item_manager_clear (PhotosItemManager *self, PhotosWindowMode mode)
 
 
 PhotosBaseItem *
-photos_item_manager_create_item (PhotosItemManager *self, TrackerSparqlCursor *cursor)
+photos_item_manager_create_item (PhotosItemManager *self,
+                                 GType base_item_type,
+                                 TrackerSparqlCursor *cursor,
+                                 gboolean create_thumbnails)
 {
-  GIOExtension *extension;
-  GType type;
+  PhotosBaseItem *item;
   PhotosBaseItem *ret_val = NULL;
-  const gchar *extension_name = "local";
-  gchar *identifier = NULL;
-  gchar **split_identifier = NULL;
+  const gchar *id;
 
   g_return_val_if_fail (PHOTOS_IS_ITEM_MANAGER (self), NULL);
+  g_return_val_if_fail (base_item_type == G_TYPE_NONE
+                        || (base_item_type != PHOTOS_TYPE_BASE_ITEM
+                            && g_type_is_a (base_item_type, PHOTOS_TYPE_BASE_ITEM)), NULL);
   g_return_val_if_fail (TRACKER_SPARQL_IS_CURSOR (cursor), NULL);
 
-  identifier = g_strdup (tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_IDENTIFIER, NULL));
-  if (identifier == NULL)
-    goto final;
-
-  split_identifier = g_strsplit (identifier, ":", 4);
-
-  if (photos_item_manager_cursor_is_collection (cursor))
+  id = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URN, NULL);
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (PHOTOS_BASE_MANAGER (self), id));
+  if (item != NULL)
     {
-      /* Its a collection. */
-      extension_name = split_identifier[2];
+      ret_val = g_object_ref (item);
     }
   else
     {
-      /* Its a normal photo item. */
-      if (g_strv_length (split_identifier) > 1)
-        extension_name = split_identifier[0];
-    }
+      GType type;
 
- final:
-  extension = g_io_extension_point_get_extension_by_name (self->extension_point, extension_name);
-  if (G_UNLIKELY (extension == NULL))
-    {
-      g_warning ("Unable to find extension %s for identifier: %s", extension_name, identifier);
-      goto out;
-    }
+      if (base_item_type == G_TYPE_NONE)
+        {
+          GIOExtension *extension;
+          g_auto (GStrv) split_identifier = NULL;
+          const gchar *extension_name = "local";
+          g_autofree gchar *identifier = NULL;
 
-  type = g_io_extension_get_type (extension);
-  ret_val = PHOTOS_BASE_ITEM (g_object_new (type,
-                                            "cursor", cursor,
-                                            "failed-thumbnailing", FALSE,
-                                            NULL));
+          identifier = g_strdup (tracker_sparql_cursor_get_string (cursor,
+                                                                   PHOTOS_QUERY_COLUMNS_IDENTIFIER,
+                                                                   NULL));
+          if (identifier != NULL)
+            {
+              split_identifier = g_strsplit (identifier, ":", 4);
+
+              if (photos_item_manager_cursor_is_collection (cursor))
+                {
+                  /* Its a collection. */
+                  extension_name = split_identifier[2];
+                }
+              else
+                {
+                  /* Its a normal photo item. */
+                  if (g_strv_length (split_identifier) > 1)
+                    extension_name = split_identifier[0];
+                }
+            }
+
+          extension = g_io_extension_point_get_extension_by_name (self->extension_point, extension_name);
+          if (G_UNLIKELY (extension == NULL))
+            {
+              g_warning ("Unable to find extension %s for identifier: %s", extension_name, identifier);
+              goto out;
+            }
+
+          type = g_io_extension_get_type (extension);
+        }
+      else
+        {
+          type = base_item_type;
+        }
+
+      g_return_val_if_fail (type != G_TYPE_NONE, NULL);
+      g_return_val_if_fail (type != PHOTOS_TYPE_BASE_ITEM && g_type_is_a (type, PHOTOS_TYPE_BASE_ITEM), NULL);
+
+      ret_val = PHOTOS_BASE_ITEM (g_object_new (type,
+                                                "cursor", cursor,
+                                                "failed-thumbnailing", !create_thumbnails,
+                                                NULL));
+    }
 
  out:
-  g_strfreev (split_identifier);
-  g_free (identifier);
   return ret_val;
 }
 
@@ -1211,6 +1530,57 @@ photos_item_manager_unhide_item (PhotosItemManager *self, PhotosBaseItem *item)
 }
 
 
+void
+photos_item_manager_wait_for_file_async (PhotosItemManager *self,
+                                         GFile *file,
+                                         GCancellable *cancellable,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data)
+{
+  GList *tasks;
+  g_autoptr (GTask) task = NULL;
+  g_autofree gchar *uri = NULL;
+
+  g_return_if_fail (PHOTOS_IS_ITEM_MANAGER (self));
+  g_return_if_fail (G_IS_FILE (file));
+  g_return_if_fail (g_file_is_native (file));
+  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_item_manager_wait_for_file_async);
+
+  uri = g_file_get_uri (file);
+  tasks = (GList *) g_hash_table_lookup (self->wait_for_changes_table, uri);
+  tasks = g_list_copy_deep (tasks, (GCopyFunc) g_object_ref, NULL);
+  tasks = g_list_prepend (tasks, g_object_ref (task));
+  g_hash_table_insert (self->wait_for_changes_table, g_strdup (uri), tasks);
+
+  photos_item_manager_remove_timeout (self);
+  self->wait_for_changes_id = g_timeout_add_seconds (WAIT_FOR_CHANGES_TIMEOUT,
+                                                     photos_item_manager_wait_for_changes_timeout,
+                                                     self);
+
+  photos_debug (PHOTOS_DEBUG_TRACKER, "Waiting for %s", uri);
+}
+
+
+gchar *
+photos_item_manager_wait_for_file_finish (PhotosItemManager *self, GAsyncResult *res, GError **error)
+{
+  GTask *task;
+
+  g_return_val_if_fail (PHOTOS_IS_ITEM_MANAGER (self), NULL);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), NULL);
+  task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_item_manager_wait_for_file_async, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  return g_task_propagate_pointer (task, error);
+}
+
+
 void
 photos_item_manager_wait_for_changes_async (PhotosItemManager *self,
                                             PhotosBaseItem *item,
@@ -1219,7 +1589,7 @@ photos_item_manager_wait_for_changes_async (PhotosItemManager *self,
                                             gpointer user_data)
 {
   GList *tasks;
-  GTask *task = NULL;
+  g_autoptr (GTask) task = NULL;
   const gchar *uri;
 
   g_return_if_fail (PHOTOS_IS_ITEM_MANAGER (self));
@@ -1244,8 +1614,15 @@ photos_item_manager_wait_for_changes_async (PhotosItemManager *self,
   tasks = g_list_prepend (tasks, g_object_ref (task));
   g_hash_table_insert (self->wait_for_changes_table, g_strdup (uri), tasks);
 
+  photos_item_manager_remove_timeout (self);
+  self->wait_for_changes_id = g_timeout_add_seconds (WAIT_FOR_CHANGES_TIMEOUT,
+                                                     photos_item_manager_wait_for_changes_timeout,
+                                                     self);
+
+  photos_debug (PHOTOS_DEBUG_TRACKER, "Waiting for %s", uri);
+
  out:
-  g_clear_object (&task);
+  return;
 }
 
 
@@ -1290,23 +1667,21 @@ photos_mode_controller_get_window_mode (PhotosModeController *self)
 void
 photos_mode_controller_go_back (PhotosModeController *self)
 {
+  GAction *selection_mode_action;
+  GApplication *app;
   PhotosWindowMode old_mode;
   PhotosWindowMode tmp;
   gboolean active_changed = FALSE;
   gboolean active_collection_changed = FALSE;
+  gboolean unset_selection_mode = FALSE;
 
   g_return_if_fail (PHOTOS_IS_MODE_CONTROLLER (self));
   g_return_if_fail (!g_queue_is_empty (self->history));
 
+  app = g_application_get_default ();
+  selection_mode_action = g_action_map_lookup_action (G_ACTION_MAP (app), "selection-mode");
+
   old_mode = (PhotosWindowMode) GPOINTER_TO_INT (g_queue_peek_head (self->history));
-
-  /* Always go back to the overview when activated from the search
-   * provider. It is easier to special case it here instead of all
-   * over the code.
-   */
-  if (self->mode == PHOTOS_WINDOW_MODE_PREVIEW && old_mode == PHOTOS_WINDOW_MODE_NONE)
-    old_mode = PHOTOS_WINDOW_MODE_OVERVIEW;
-
   g_return_if_fail (old_mode != PHOTOS_WINDOW_MODE_NONE);
 
   switch (self->mode)
@@ -1330,6 +1705,20 @@ photos_mode_controller_go_back (PhotosModeController *self)
       g_return_if_fail (old_mode != PHOTOS_WINDOW_MODE_PREVIEW);
       break;
 
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      {
+        g_autoptr (GVariant) state = NULL;
+
+        g_return_if_fail (old_mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+                          || old_mode == PHOTOS_WINDOW_MODE_FAVORITES
+                          || old_mode == PHOTOS_WINDOW_MODE_OVERVIEW);
+
+        state = g_action_get_state (selection_mode_action);
+        g_return_if_fail (state != NULL);
+        g_return_if_fail (g_variant_get_boolean (state));
+        break;
+      }
+
     case PHOTOS_WINDOW_MODE_PREVIEW:
       g_return_if_fail (PHOTOS_IS_BASE_ITEM (self->active_object));
       g_return_if_fail (self->active_object != (GObject *) self->active_collection);
@@ -1366,6 +1755,10 @@ photos_mode_controller_go_back (PhotosModeController *self)
     case PHOTOS_WINDOW_MODE_EDIT:
       break;
 
+    case PHOTOS_WINDOW_MODE_IMPORT:
+      unset_selection_mode = TRUE;
+      break;
+
     case PHOTOS_WINDOW_MODE_PREVIEW:
       self->load_state = PHOTOS_LOAD_STATE_NONE;
       g_set_object (&self->active_object, G_OBJECT (self->active_collection));
@@ -1394,6 +1787,14 @@ photos_mode_controller_go_back (PhotosModeController *self)
       break;
     }
 
+  if (unset_selection_mode)
+    {
+      GVariant *state;
+
+      state = g_variant_new_boolean (FALSE);
+      g_action_change_state (selection_mode_action, state);
+    }
+
   if (active_changed)
     g_signal_emit_by_name (self, "active-changed", self->active_object);
 
@@ -1428,21 +1829,41 @@ photos_mode_controller_set_fullscreen (PhotosModeController *self, gboolean full
 void
 photos_mode_controller_set_window_mode (PhotosModeController *self, PhotosWindowMode mode)
 {
+  GAction *selection_mode_action;
+  GApplication *app;
   PhotosWindowMode old_mode;
+  gboolean active_changed = FALSE;
   gboolean active_collection_changed = FALSE;
+  gboolean set_selection_mode = FALSE;
 
   g_return_if_fail (PHOTOS_IS_MODE_CONTROLLER (self));
   g_return_if_fail (mode != PHOTOS_WINDOW_MODE_NONE);
   g_return_if_fail (mode != PHOTOS_WINDOW_MODE_COLLECTION_VIEW);
   g_return_if_fail (mode != PHOTOS_WINDOW_MODE_PREVIEW);
 
+  app = g_application_get_default ();
+  selection_mode_action = g_action_map_lookup_action (G_ACTION_MAP (app), "selection-mode");
+
   if (mode == PHOTOS_WINDOW_MODE_EDIT)
     {
       g_return_if_fail (self->load_state == PHOTOS_LOAD_STATE_FINISHED);
       g_return_if_fail (self->mode == PHOTOS_WINDOW_MODE_PREVIEW);
     }
+  else if (mode == PHOTOS_WINDOW_MODE_IMPORT)
+    {
+      g_autoptr (GVariant) state = NULL;
+
+      g_return_if_fail (self->mode == PHOTOS_WINDOW_MODE_COLLECTIONS
+                        || self->mode == PHOTOS_WINDOW_MODE_FAVORITES
+                        || self->mode == PHOTOS_WINDOW_MODE_OVERVIEW);
+
+      state = g_action_get_state (selection_mode_action);
+      g_return_if_fail (state != NULL);
+      g_return_if_fail (!g_variant_get_boolean (state));
+    }
   else
     {
+      g_return_if_fail (self->mode != PHOTOS_WINDOW_MODE_EDIT);
       g_return_if_fail (self->mode != PHOTOS_WINDOW_MODE_PREVIEW);
     }
 
@@ -1463,11 +1884,25 @@ photos_mode_controller_set_window_mode (PhotosModeController *self, PhotosWindow
         }
 
       g_clear_object (&self->active_object);
-      g_signal_emit_by_name (self, "active-changed", self->active_object);
-
-      if (active_collection_changed)
-        g_signal_emit (self, signals[ACTIVE_COLLECTION_CHANGED], 0, self->active_collection);
+      active_changed = TRUE;
     }
 
+  if (mode == PHOTOS_WINDOW_MODE_IMPORT)
+    set_selection_mode = TRUE;
+
+  if (set_selection_mode)
+    {
+      GVariant *state;
+
+      state = g_variant_new_boolean (TRUE);
+      g_action_change_state (selection_mode_action, state);
+    }
+
+  if (active_changed)
+    g_signal_emit_by_name (self, "active-changed", self->active_object);
+
+  if (active_collection_changed)
+    g_signal_emit (self, signals[ACTIVE_COLLECTION_CHANGED], 0, self->active_collection);
+
   g_signal_emit (self, signals[WINDOW_MODE_CHANGED], 0, mode, old_mode);
 }
