@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -62,6 +60,7 @@ typedef enum
   PHOTOS_WINDOW_MODE_COLLECTIONS,
   PHOTOS_WINDOW_MODE_EDIT,
   PHOTOS_WINDOW_MODE_FAVORITES,
+  PHOTOS_WINDOW_MODE_IMPORT,
   PHOTOS_WINDOW_MODE_OVERVIEW,
   PHOTOS_WINDOW_MODE_PREVIEW,
   PHOTOS_WINDOW_MODE_SEARCH
@@ -73,10 +72,12 @@ typedef struct _PhotosItemManagerClass PhotosModeControllerClass;
 PhotosBaseManager        *photos_item_manager_new                          (void);
 
 void                      photos_item_manager_add_item                     (PhotosItemManager *self,
+                                                                            GType base_item_type,
                                                                             TrackerSparqlCursor *cursor,
                                                                             gboolean force);
 
 void                      photos_item_manager_add_item_for_mode            (PhotosItemManager *self,
+                                                                            GType base_item_type,
                                                                             PhotosWindowMode mode,
                                                                             TrackerSparqlCursor *cursor);
 
@@ -84,7 +85,9 @@ void                      photos_item_manager_clear                        (Phot
                                                                             PhotosWindowMode mode);
 
 PhotosBaseItem           *photos_item_manager_create_item                  (PhotosItemManager *self,
-                                                                            TrackerSparqlCursor *cursor);
+                                                                            GType base_item_type,
+                                                                            TrackerSparqlCursor *cursor,
+                                                                            gboolean create_thumbnails);
 
 PhotosBaseItem           *photos_item_manager_get_active_collection        (PhotosItemManager *self);
 
@@ -102,6 +105,16 @@ void                      photos_item_manager_set_constraints_for_mode     (Phot
                                                                             gboolean constrain,
                                                                             PhotosWindowMode mode);
 
+void                      photos_item_manager_wait_for_file_async          (PhotosItemManager *self,
+                                                                            GFile *file,
+                                                                            GCancellable *cancellable,
+                                                                            GAsyncReadyCallback callback,
+                                                                            gpointer user_data);
+
+gchar                    *photos_item_manager_wait_for_file_finish         (PhotosItemManager *self,
+                                                                            GAsyncResult *res,
+                                                                            GError **error);
+
 void                      photos_item_manager_wait_for_changes_async       (PhotosItemManager *self,
                                                                             PhotosBaseItem *item,
                                                                             GCancellable *cancellable,
