@@ -67,9 +67,7 @@ static void photos_main_toolbar_favorite_button_update (PhotosMainToolbar *self,
 
 
 static gchar *
-photos_main_toolbar_create_selection_mode_label (PhotosMainToolbar *self,
-                                                 PhotosBaseItem *active_collection,
-                                                 PhotosSource *source)
+photos_main_toolbar_create_selection_mode_label (PhotosMainToolbar *self, PhotosBaseItem *active_collection)
 {
   GList *selection;
   g_autofree gchar *label = NULL;
@@ -84,20 +82,9 @@ photos_main_toolbar_create_selection_mode_label (PhotosMainToolbar *self,
     label = g_strdup_printf (ngettext ("%d selected", "%d selected", length), length);
 
   if (active_collection != NULL)
-    {
-      ret_val = g_markup_printf_escaped ("<b>%s</b> (%s)", photos_base_item_get_name (active_collection), label);
-    }
-  else if (source != NULL)
-    {
-      const gchar *name;
-
-      name = photos_source_get_name (source);
-      ret_val = g_markup_printf_escaped ("<b>%s</b> (%s)", name, label);
-    }
+    ret_val = g_markup_printf_escaped ("<b>%s</b> (%s)", photos_base_item_get_name (active_collection), label);
   else
-    {
-      ret_val = g_steal_pointer (&label);
-    }
+    ret_val = g_steal_pointer (&label);
 
   return ret_val;
 }
@@ -146,7 +133,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
     {
     case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
       if (selection_mode)
-        primary = photos_main_toolbar_create_selection_mode_label (self, active_collection, NULL);
+        primary = photos_main_toolbar_create_selection_mode_label (self, active_collection);
       else
         primary = g_strdup (photos_base_item_get_name (active_collection));
       break;
@@ -156,7 +143,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
     case PHOTOS_WINDOW_MODE_OVERVIEW:
     case PHOTOS_WINDOW_MODE_SEARCH:
       if (selection_mode)
-        primary = photos_main_toolbar_create_selection_mode_label (self, NULL, NULL);
+        primary = photos_main_toolbar_create_selection_mode_label (self, NULL);
       break;
 
     case PHOTOS_WINDOW_MODE_EDIT:
@@ -172,8 +159,26 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       }
 
     case PHOTOS_WINDOW_MODE_IMPORT:
-      primary = photos_main_toolbar_create_selection_mode_label (self, NULL, source);
-      break;
+      {
+        GList *selection;
+        guint length;
+
+        selection = photos_selection_controller_get_selection (self->sel_cntrlr);
+        length = g_list_length (selection);
+        if (length == 0)
+          {
+            primary = g_strdup (_("Select items for import"));
+          }
+        else
+          {
+            primary = g_strdup_printf (ngettext ("Select items for import (%u selected)",
+                                                 "Select items for import (%u selected)",
+                                                 length),
+                                       length);
+          }
+
+        break;
+      }
 
     case PHOTOS_WINDOW_MODE_NONE:
     default:
@@ -460,7 +465,7 @@ photos_main_toolbar_populate_for_edit (PhotosMainToolbar *self)
   GtkWidget *cancel_button;
   GtkWidget *done_button;
 
-  cancel_button = gtk_button_new_with_label (_("Cancel"));
+  cancel_button = gtk_button_new_with_mnemonic (_("_Cancel"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (cancel_button), "app.edit-cancel");
   gtk_header_bar_pack_start (GTK_HEADER_BAR (self->header_bar), cancel_button);
 
@@ -489,14 +494,19 @@ photos_main_toolbar_populate_for_import (PhotosMainToolbar *self)
 {
   GtkStyleContext *context;
   GtkWidget *cancel_button;
+  GtkWidget *select_button;
 
   gtk_header_bar_set_custom_title (GTK_HEADER_BAR (self->header_bar), self->selection_menu);
   context = gtk_widget_get_style_context (self->header_bar);
   gtk_style_context_add_class (context, "selection-mode");
 
-  cancel_button = gtk_button_new_with_label (_("Cancel"));
+  cancel_button = gtk_button_new_with_mnemonic (_("_Cancel"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (cancel_button), "app.import-cancel");
-  gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header_bar), cancel_button);
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (self->header_bar), cancel_button);
+
+  select_button = gtk_button_new_with_mnemonic (_("_Select"));
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (select_button), "app.import-current");
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header_bar), select_button);
 
   g_signal_connect_object (self->sel_cntrlr,
                            "selection-changed",
@@ -601,7 +611,7 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
   context = gtk_widget_get_style_context (self->header_bar);
   gtk_style_context_add_class (context, "selection-mode");
 
-  selection_button = gtk_button_new_with_label (_("Cancel"));
+  selection_button = gtk_button_new_with_mnemonic (_("_Cancel"));
   gtk_actionable_set_action_name (GTK_ACTIONABLE (selection_button), "app.selection-mode");
   gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header_bar), selection_button);
 
