@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -43,6 +41,7 @@ struct _PhotosMainWindow
 {
   GtkApplicationWindow parent_instance;
   GAction *edit_cancel;
+  GAction *import_cancel;
   GAction *load_next;
   GAction *load_previous;
   GtkWidget *embed;
@@ -168,6 +167,7 @@ photos_main_window_go_back (PhotosMainWindow *self)
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
     case PHOTOS_WINDOW_MODE_EDIT:
     case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_IMPORT:
     case PHOTOS_WINDOW_MODE_OVERVIEW:
     case PHOTOS_WINDOW_MODE_SEARCH:
     default:
@@ -205,21 +205,6 @@ photos_main_window_handle_back_key (PhotosMainWindow *self, GdkEventKey *event)
 }
 
 
-static gboolean
-photos_main_window_handle_key_edit (PhotosMainWindow *self, GdkEventKey *event)
-{
-  gboolean handled = FALSE;
-
-  if (event->keyval == GDK_KEY_Escape)
-    {
-      g_action_activate (self->edit_cancel, NULL);
-      handled = TRUE;
-    }
-
-  return handled;
-}
-
-
 static gboolean
 photos_main_window_handle_key_preview (PhotosMainWindow *self, GdkEventKey *event)
 {
@@ -277,16 +262,14 @@ photos_main_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
     case PHOTOS_WINDOW_MODE_NONE:
     case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
+    case PHOTOS_WINDOW_MODE_EDIT:
     case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_IMPORT:
     case PHOTOS_WINDOW_MODE_OVERVIEW:
     case PHOTOS_WINDOW_MODE_SEARCH:
       handled = GDK_EVENT_PROPAGATE;
       break;
 
-    case PHOTOS_WINDOW_MODE_EDIT:
-      handled = photos_main_window_handle_key_edit (self, event);
-      break;
-
     case PHOTOS_WINDOW_MODE_PREVIEW:
       handled = photos_main_window_handle_key_preview (self, event);
       break;
@@ -352,6 +335,7 @@ photos_main_window_constructed (GObject *object)
   gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (self));
 
   self->edit_cancel = g_action_map_lookup_action (G_ACTION_MAP (app), "edit-cancel");
+  self->import_cancel = g_action_map_lookup_action (G_ACTION_MAP (app), "import-cancel");
   self->load_next = g_action_map_lookup_action (G_ACTION_MAP (app), "load-next");
   self->load_previous = g_action_map_lookup_action (G_ACTION_MAP (app), "load-previous");
 
