@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -29,6 +27,7 @@
 #include <tracker-sparql.h>
 
 #include "egg-counter.h"
+#include "photos-debug.h"
 #include "photos-offset-controller.h"
 #include "photos-query-builder.h"
 #include "photos-tracker-queue.h"
@@ -61,7 +60,7 @@ EGG_DEFINE_COUNTER (instances,
 
 enum
 {
-  OFFSET_STEP = 50
+  OFFSET_STEP = 60
 };
 
 
@@ -92,7 +91,13 @@ photos_offset_controller_cursor_next (GObject *source_object, GAsyncResult *res,
 
   if (success)
     {
+      const gchar *type_name;
+
       priv->count = (gint) tracker_sparql_cursor_get_integer (cursor, 0);
+
+      type_name = G_OBJECT_TYPE_NAME (self);
+      photos_debug (PHOTOS_DEBUG_TRACKER, "%s has %d items", type_name, priv->count);
+
       g_signal_emit (self, signals[COUNT_CHANGED], 0, priv->count);
     }
 
