@@ -35,16 +35,16 @@ struct _PhotosPipeline
 {
   GObject parent_instance;
   GHashTable *hash;
+  GStrv uris;
   GeglNode *graph;
   gchar *snapshot;
-  gchar *uri;
 };
 
 enum
 {
   PROP_0,
   PROP_PARENT,
-  PROP_URI
+  PROP_URIS
 };
 
 static void photos_pipeline_async_initable_iface_init (GAsyncInitableIface *iface);
@@ -126,10 +126,10 @@ photos_pipeline_reset (PhotosPipeline *self)
 static gboolean
 photos_pipeline_create_graph_from_xml (PhotosPipeline *self, const gchar *contents)
 {
-  GeglNode *graph = NULL;
+  g_autoptr (GeglNode) graph = NULL;
   GeglNode *input;
   GeglNode *output;
-  GSList *children = NULL;
+  g_autoptr (GSList) children = NULL;
   GSList *l;
   gboolean ret_val = FALSE;
 
@@ -175,8 +175,6 @@ photos_pipeline_create_graph_from_xml (PhotosPipeline *self, const gchar *conten
   ret_val = TRUE;
 
  out:
-  g_slist_free (children);
-  g_clear_object (&graph);
   return ret_val;
 }
 
@@ -184,21 +182,23 @@ photos_pipeline_create_graph_from_xml (PhotosPipeline *self, const gchar *conten
 static void
 photos_pipeline_save_replace_contents (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  GTask *task = G_TASK (user_data);
-  GError *error;
+  g_autoptr (GTask) task = G_TASK (user_data);
   GFile *file = G_FILE (source_object);
 
-  error = NULL;
-  if (!g_file_replace_contents_finish (file, res, NULL, &error))
-    {
-      g_task_return_error (task, error);
-      goto out;
-    }
+  {
+    g_autoptr (GError) error = NULL;
+
+    if (!g_file_replace_contents_finish (file, res, NULL, &error))
+      {
+        g_task_return_error (task, g_steal_pointer (&error));
+        goto out;
+      }
+  }
 
   g_task_return_boolean (task, TRUE);
 
  out:
-  g_object_unref (task);
+  return;
 }
 
 
@@ -242,8 +242,8 @@ photos_pipeline_finalize (GObject *object)
 {
   PhotosPipeline *self = PHOTOS_PIPELINE (object);
 
+  g_strfreev (self->uris);
   g_free (self->snapshot);
-  g_free (self->uri);
 
   G_OBJECT_CLASS (photos_pipeline_parent_class)->finalize (object);
 
@@ -267,8 +267,8 @@ photos_pipeline_set_property (GObject *object, guint prop_id, const GValue *valu
         break;
       }
 
-    case PROP_URI:
-      self->uri = g_value_dup_string (value);
+    case PROP_URIS:
+      self->uris = (GStrv) g_value_dup_boxed (value);
       break;
 
     default:
@@ -307,56 +307,78 @@ photos_pipeline_class_init (PhotosPipelineClass *class)
                                                         G_PARAM_CONSTRUCT | G_PARAM_WRITABLE));
 
   g_object_class_install_property (object_class,
-                                   PROP_URI,
-                                   g_param_spec_string ("uri",
-                                                        "An URI",
-                                                        "The location to save this pipeline",
-                                                        NULL,
-                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+                                   PROP_URIS,
+                                   g_param_spec_boxed ("uris",
+                                                       "URIs",
+                                                       "Possible locations from which to load this pipeline, and"
+                                                       "afterwards it will be saved to the first non-NULL URI.",
+                                                       G_TYPE_STRV,
+                                                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 }
 
 
 static void
 photos_pipeline_async_initable_init_load_contents (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  GTask *task = G_TASK (user_data);
+  g_autoptr (GTask) task = G_TASK (user_data);
   PhotosPipeline *self;
-  GError *error;
+  GCancellable *cancellable;
   GFile *file = G_FILE (source_object);
-  gchar *contents = NULL;
+  g_autofree gchar *contents = NULL;
+  const gchar *uri;
 
   self = PHOTOS_PIPELINE (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+  uri = (const gchar *) g_task_get_task_data (task);
 
-  error = NULL;
-  if (!g_file_load_contents_finish (file, res, &contents, NULL, NULL, &error))
-    {
-      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-        {
-          g_error_free (error);
-          goto carry_on;
-        }
-      else
-        {
-          g_task_return_error (task, error);
-          goto out;
-        }
-    }
+  {
+    g_autoptr (GError) error = NULL;
+
+    if (!g_file_load_contents_finish (file, res, &contents, NULL, NULL, &error))
+      {
+        if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+          {
+            g_autoptr (GFile) file_next = NULL;
+            guint i;
+
+            for (i = 0; self->uris[i] != NULL; i++)
+              {
+                if (g_strcmp0 (self->uris[i], uri) == 0)
+                  break;
+              }
+
+            g_assert_nonnull (self->uris[i]);
+
+            i++;
+            if (self->uris[i] == NULL)
+              goto carry_on;
+
+            g_task_set_task_data (task, g_strdup (self->uris[i]), g_free);
+
+            file_next = g_file_new_for_uri (self->uris[i]);
+            g_file_load_contents_async (file_next,
+                                        cancellable,
+                                        photos_pipeline_async_initable_init_load_contents,
+                                        g_object_ref (task));
+
+            goto out;
+          }
+        else
+          {
+            g_task_return_error (task, g_steal_pointer (&error));
+            goto out;
+          }
+      }
+  }
 
   if (!(photos_pipeline_create_graph_from_xml (self, contents)))
-    {
-      gchar *uri;
-
-      uri = g_file_get_uri (file);
-      g_warning ("Unable to deserialize from %s", uri);
-      g_free (uri);
-    }
+    g_warning ("Unable to deserialize from %s", uri);
 
  carry_on:
   g_task_return_boolean (task, TRUE);
 
  out:
-  g_free (contents);
-  g_object_unref (task);
+  return;
 }
 
 
@@ -368,27 +390,28 @@ photos_pipeline_async_initable_init_async (GAsyncInitable *initable,
                                            gpointer user_data)
 {
   PhotosPipeline *self = PHOTOS_PIPELINE (initable);
-  GFile *file = NULL;
-  GTask *task = NULL;
+  g_autoptr (GFile) file = NULL;
+  g_autoptr (GTask) task = NULL;
 
   task = g_task_new (self, cancellable, callback, user_data);
   g_task_set_source_tag (task, photos_pipeline_async_initable_init_async);
 
-  if (self->uri == NULL)
+  if (self->uris == NULL || self->uris[0] == NULL || self->uris[0][0] == '\0')
     {
       g_task_return_boolean (task, TRUE);
       goto out;
     }
 
-  file = g_file_new_for_uri (self->uri);
+  g_task_set_task_data (task, g_strdup (self->uris[0]), g_free);
+
+  file = g_file_new_for_uri (self->uris[0]);
   g_file_load_contents_async (file,
                               cancellable,
                               photos_pipeline_async_initable_init_load_contents,
                               g_object_ref (task));
 
  out:
-  g_clear_object (&file);
-  g_object_unref (task);
+  return;
 }
 
 
@@ -416,7 +439,7 @@ photos_pipeline_async_initable_iface_init (GAsyncInitableIface *iface)
 
 void
 photos_pipeline_new_async (GeglNode *parent,
-                           const gchar *uri,
+                           const gchar *const *uris,
                            GCancellable *cancellable,
                            GAsyncReadyCallback callback,
                            gpointer user_data)
@@ -430,7 +453,7 @@ photos_pipeline_new_async (GeglNode *parent,
                               callback,
                               user_data,
                               "parent", parent,
-                              "uri", uri,
+                              "uris", uris,
                               NULL);
 }
 
@@ -439,13 +462,12 @@ PhotosPipeline *
 photos_pipeline_new_finish (GAsyncResult *res, GError **error)
 {
   GObject *ret_val;
-  GObject *source_object;
+  g_autoptr (GObject) source_object = NULL;
 
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
   source_object = g_async_result_get_source_object (res);
   ret_val = g_async_initable_new_finish (G_ASYNC_INITABLE (source_object), res, error);
-  g_object_unref (source_object);
   return PHOTOS_PIPELINE (ret_val);
 }
 
@@ -460,7 +482,7 @@ photos_pipeline_add_valist (PhotosPipeline *self,
   GeglNode *last;
   GeglNode *node;
   GeglNode *output;
-  gchar *xml = NULL;
+  g_autofree gchar *xml = NULL;
 
   g_return_if_fail (PHOTOS_IS_PIPELINE (self));
   g_return_if_fail (operation != NULL && operation[0] != '\0');
@@ -489,8 +511,6 @@ photos_pipeline_add_valist (PhotosPipeline *self,
 
   xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
   photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
-
-  g_free (xml);
 }
 
 
@@ -561,7 +581,7 @@ photos_pipeline_get_output (PhotosPipeline *self)
 gboolean
 photos_pipeline_is_edited (PhotosPipeline *self)
 {
-  GSList *children = NULL;
+  g_autoptr (GSList) children = NULL;
   GSList *l;
   guint n_operations = 0;
 
@@ -598,7 +618,6 @@ photos_pipeline_is_edited (PhotosPipeline *self)
     }
 
  out:
-  g_slist_free (children);
   return n_operations > 0;
 }
 
@@ -621,8 +640,8 @@ photos_pipeline_save_async (PhotosPipeline *self,
                             GAsyncReadyCallback callback,
                             gpointer user_data)
 {
-  GFile *file;
-  GTask *task;
+  g_autoptr (GFile) file = NULL;
+  g_autoptr (GTask) task = NULL;
   gchar *xml = NULL;
   gsize len;
 
@@ -640,7 +659,7 @@ photos_pipeline_save_async (PhotosPipeline *self,
    */
   g_task_set_task_data (task, xml, g_free);
 
-  file = g_file_new_for_uri (self->uri);
+  file = g_file_new_for_uri (self->uris[0]);
   len = strlen (xml);
   g_file_replace_contents_async (file,
                                  xml,
@@ -651,9 +670,6 @@ photos_pipeline_save_async (PhotosPipeline *self,
                                  cancellable,
                                  photos_pipeline_save_replace_contents,
                                  g_object_ref (task));
-
-  g_object_unref (file);
-  g_object_unref (task);
 }
 
 
@@ -675,7 +691,7 @@ photos_pipeline_remove (PhotosPipeline *self, const gchar *operation)
 {
   GeglNode *node;
   gboolean ret_val = FALSE;
-  gchar *xml = NULL;
+  g_autofree gchar *xml = NULL;
 
   g_return_val_if_fail (PHOTOS_IS_PIPELINE (self), FALSE);
   g_return_val_if_fail (operation != NULL && operation[0] != '\0', FALSE);
@@ -695,7 +711,6 @@ photos_pipeline_remove (PhotosPipeline *self, const gchar *operation)
   ret_val = TRUE;
 
  out:
-  g_free (xml);
   return ret_val;
 }
 
@@ -703,7 +718,7 @@ photos_pipeline_remove (PhotosPipeline *self, const gchar *operation)
 void
 photos_pipeline_revert (PhotosPipeline *self)
 {
-  gchar *xml;
+  g_autofree gchar *xml = NULL;
 
   g_return_if_fail (PHOTOS_IS_PIPELINE (self));
   g_return_if_fail (self->snapshot != NULL);
@@ -715,8 +730,6 @@ photos_pipeline_revert (PhotosPipeline *self)
 
   xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
   photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
-
-  g_free (xml);
 }
 
 
@@ -724,7 +737,7 @@ void
 photos_pipeline_revert_to_original (PhotosPipeline *self)
 {
   const gchar *empty_xml = "<?xml version='1.0' encoding='UTF-8'?><gegl></gegl>";
-  gchar *xml;
+  g_autofree gchar *xml = NULL;
 
   g_return_if_fail (PHOTOS_IS_PIPELINE (self));
 
@@ -735,8 +748,6 @@ photos_pipeline_revert_to_original (PhotosPipeline *self)
 
   xml = gegl_node_to_xml_full (self->graph, self->graph, "/");
   photos_debug (PHOTOS_DEBUG_GEGL, "Pipeline: %s", xml);
-
-  g_free (xml);
 }
 
 
