@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -33,7 +31,9 @@
 
 G_BEGIN_DECLS
 
-PhotosQuery  *photos_query_builder_create_collection_query (PhotosSearchContextState *state, const gchar *name);
+PhotosQuery  *photos_query_builder_create_collection_query (PhotosSearchContextState *state,
+                                                            const gchar *name,
+                                                            const gchar *identifier_tag);
 
 PhotosQuery  *photos_query_builder_collection_icon_query (PhotosSearchContextState *state, const gchar *resource);
 
@@ -43,7 +43,10 @@ PhotosQuery  *photos_query_builder_delete_resource_query (PhotosSearchContextSta
 
 PhotosQuery  *photos_query_builder_equipment_query (PhotosSearchContextState *state, GQuark equipment);
 
-PhotosQuery  *photos_query_builder_fetch_collections_query (PhotosSearchContextState *state, const gchar *resource);
+PhotosQuery  *photos_query_builder_fetch_collections_for_urn_query (PhotosSearchContextState *state,
+                                                                    const gchar *resource);
+
+PhotosQuery  *photos_query_builder_fetch_collections_local (PhotosSearchContextState *state);
 
 PhotosQuery  *photos_query_builder_global_query        (PhotosSearchContextState *state,
                                                         gint flags,
@@ -62,8 +65,6 @@ PhotosQuery  *photos_query_builder_single_query        (PhotosSearchContextState
 
 PhotosQuery  *photos_query_builder_update_mtime_query (PhotosSearchContextState *state, const gchar *resource);
 
-gchar        *photos_query_builder_filter_local        (void);
-
 G_END_DECLS
 
 #endif /* PHOTOS_QUERY_BUILDER */
