@@ -44,15 +44,8 @@ struct _PhotosSelectionToolbar
   GAction *selection_mode_action;
   GHashTable *item_listeners;
   GtkWidget *toolbar_collection;
-  GtkWidget *toolbar_export;
   GtkWidget *toolbar_favorite;
-  GtkWidget *toolbar_import;
   GtkWidget *toolbar_open;
-  GtkWidget *toolbar_print;
-  GtkWidget *toolbar_properties;
-  GtkWidget *toolbar_separator;
-  GtkWidget *toolbar_share;
-  GtkWidget *toolbar_trash;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
   PhotosSelectionController *sel_cntrlr;
@@ -199,20 +192,9 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
   GList *l;
   GList *selection;
   GtkWidget *image;
-  PhotosWindowMode mode;
   gboolean has_selection;
   gboolean enable_collection;
   gboolean enable_favorite;
-  gboolean show_collection;
-  gboolean show_export;
-  gboolean show_favorite;
-  gboolean show_import;
-  gboolean show_open;
-  gboolean show_print;
-  gboolean show_properties;
-  gboolean show_separator;
-  gboolean show_share;
-  gboolean show_trash;
   gchar *favorite_label;
   gchar *open_label;
   guint fav_count = 0;
@@ -220,47 +202,6 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
 
   self->inside_refresh = TRUE;
 
-  mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
-  switch (mode)
-    {
-    case PHOTOS_WINDOW_MODE_COLLECTION_VIEW:
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      show_collection = TRUE;
-      show_export = TRUE;
-      show_favorite = TRUE;
-      show_import = FALSE;
-      show_open = TRUE;
-      show_print = TRUE;
-      show_properties = TRUE;
-      show_separator = TRUE;
-      show_share = TRUE;
-      show_trash = TRUE;
-      break;
-
-    case PHOTOS_WINDOW_MODE_IMPORT:
-      show_collection = FALSE;
-      show_export = FALSE;
-      show_favorite = FALSE;
-      show_import = TRUE;
-      show_open = FALSE;
-      show_print = FALSE;
-      show_properties = FALSE;
-      show_separator = FALSE;
-      show_share = FALSE;
-      show_trash = FALSE;
-      break;
-
-    case PHOTOS_WINDOW_MODE_NONE:
-    case PHOTOS_WINDOW_MODE_EDIT:
-    case PHOTOS_WINDOW_MODE_PREVIEW:
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-
   selection = photos_selection_controller_get_selection (self->sel_cntrlr);
   has_selection = selection != NULL;
 
@@ -320,17 +261,6 @@ photos_selection_toolbar_set_item_visibility (PhotosSelectionToolbar *self)
   gtk_widget_set_sensitive (self->toolbar_collection, enable_collection);
   gtk_widget_set_sensitive (self->toolbar_favorite, enable_favorite);
 
-  gtk_widget_set_visible (self->toolbar_collection, show_collection);
-  gtk_widget_set_visible (self->toolbar_export, show_export);
-  gtk_widget_set_visible (self->toolbar_favorite, show_favorite);
-  gtk_widget_set_visible (self->toolbar_import, show_import);
-  gtk_widget_set_visible (self->toolbar_open, show_open);
-  gtk_widget_set_visible (self->toolbar_print, show_print);
-  gtk_widget_set_visible (self->toolbar_properties, show_properties);
-  gtk_widget_set_visible (self->toolbar_separator, show_separator);
-  gtk_widget_set_visible (self->toolbar_share, show_share);
-  gtk_widget_set_visible (self->toolbar_trash, show_trash);
-
   self->inside_refresh = FALSE;
 }
 
@@ -363,10 +293,15 @@ static void
 photos_selection_toolbar_selection_changed (PhotosSelectionToolbar *self)
 {
   GList *selection;
+  PhotosWindowMode mode;
 
   if (!photos_utils_get_selection_mode ())
     return;
 
+  mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+  if (mode == PHOTOS_WINDOW_MODE_IMPORT)
+    return;
+
   selection = photos_selection_controller_get_selection (self->sel_cntrlr);
   photos_selection_toolbar_set_item_listeners (self, selection);
 
@@ -378,7 +313,11 @@ photos_selection_toolbar_selection_changed (PhotosSelectionToolbar *self)
 static void
 photos_selection_toolbar_selection_mode_notify_state (PhotosSelectionToolbar *self)
 {
-  if (photos_utils_get_selection_mode ())
+  PhotosWindowMode mode;
+
+  mode = photos_mode_controller_get_window_mode (self->mode_cntrlr);
+
+  if (photos_utils_get_selection_mode () && mode != PHOTOS_WINDOW_MODE_IMPORT)
     photos_selection_toolbar_selection_changed (self);
   else
     gtk_widget_hide (GTK_WIDGET (self));
@@ -448,15 +387,8 @@ photos_selection_toolbar_class_init (PhotosSelectionToolbarClass *class)
   object_class->dispose = photos_selection_toolbar_dispose;
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Photos/selection-toolbar.ui");
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_export);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_favorite);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_import);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_open);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_print);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_properties);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_separator);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_share);
-  gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_trash);
   gtk_widget_class_bind_template_child (widget_class, PhotosSelectionToolbar, toolbar_collection);
   gtk_widget_class_bind_template_callback (widget_class, photos_selection_toolbar_favorite_clicked);
   gtk_widget_class_bind_template_callback (widget_class, photos_selection_toolbar_collection_clicked);
