@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2016 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -43,12 +41,13 @@ enum
   PROP_SOURCE,
 };
 
-static void photos_filterable_interface_init (PhotosFilterableInterface *iface);
+static void photos_share_point_online_filterable_iface_init (PhotosFilterableInterface *iface);
 
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PhotosSharePointOnline, photos_share_point_online, PHOTOS_TYPE_SHARE_POINT,
                                   G_ADD_PRIVATE (PhotosSharePointOnline)
-                                  G_IMPLEMENT_INTERFACE (PHOTOS_TYPE_FILTERABLE, photos_filterable_interface_init));
+                                  G_IMPLEMENT_INTERFACE (PHOTOS_TYPE_FILTERABLE,
+                                                         photos_share_point_online_filterable_iface_init));
 
 
 static GIcon *
@@ -84,6 +83,13 @@ photos_share_point_online_get_name (PhotosSharePoint *share_point)
 }
 
 
+static gboolean
+photos_share_point_online_is_search_criterion (PhotosFilterable *filterable)
+{
+  return FALSE;
+}
+
+
 static void
 photos_share_point_online_dispose (GObject *object)
 {
@@ -203,9 +209,10 @@ photos_share_point_online_class_init (PhotosSharePointOnlineClass *class)
 
 
 static void
-photos_filterable_interface_init (PhotosFilterableInterface *iface)
+photos_share_point_online_filterable_iface_init (PhotosFilterableInterface *iface)
 {
   iface->get_id = photos_share_point_online_get_id;
+  iface->is_search_criterion = photos_share_point_online_is_search_criterion;
 }
 
 
