@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -25,18 +23,19 @@
 
 #include "config.h"
 
-#include <gio/gio.h>
-
 #include "egg-counter.h"
 #include "photos-filterable.h"
-#include "photos-query-builder.h"
+#include "photos-query.h"
 #include "photos-source.h"
+#include "photos-utils.h"
 
 
 struct _PhotosSource
 {
   GObject parent_instance;
   GIcon *icon;
+  GIcon *symbolic_icon;
+  GMount *mount;
   GoaObject *object;
   gboolean builtin;
   gchar *id;
@@ -48,6 +47,7 @@ enum
   PROP_0,
   PROP_BUILTIN,
   PROP_ID,
+  PROP_MOUNT,
   PROP_NAME,
   PROP_OBJECT
 };
@@ -61,17 +61,99 @@ G_DEFINE_TYPE_WITH_CODE (PhotosSource, photos_source, G_TYPE_OBJECT,
 EGG_DEFINE_COUNTER (instances, "PhotosSource", "Instances", "Number of PhotosSource instances")
 
 
+static const gchar *TRACKER_SCHEMA = "org.freedesktop.Tracker.Miner.Files";
+static const gchar *TRACKER_KEY_RECURSIVE_DIRECTORIES = "index-recursive-directories";
+
+
+static gchar *
+photos_source_build_filter_local (void)
+{
+  g_autoptr (GSettings) settings = NULL;
+  g_autoptr (GString) tracker_filter = NULL;
+  g_auto (GStrv) tracker_dirs = NULL;
+  g_autofree gchar *desktop_uri = NULL;
+  g_autofree gchar *download_uri = NULL;
+  g_autofree gchar *export_path = NULL;
+  g_autofree gchar *export_uri = NULL;
+  gchar *filter;
+  const gchar *path;
+  g_autofree gchar *pictures_uri = NULL;
+  guint i;
+
+  settings = g_settings_new (TRACKER_SCHEMA);
+  tracker_dirs = g_settings_get_strv (settings, TRACKER_KEY_RECURSIVE_DIRECTORIES);
+  tracker_filter = g_string_new ("");
+
+  for (i = 0; tracker_dirs[i] != NULL; i++)
+    {
+      g_autofree gchar *tracker_uri = NULL;
+
+      /* ignore special XDG placeholders, since we handle those internally */
+      if (tracker_dirs[i][0] == '&' || tracker_dirs[i][0] == '$')
+        continue;
+
+      tracker_uri = photos_utils_convert_path_to_uri (tracker_dirs[i]);
+      g_string_append_printf (tracker_filter, " || fn:contains (nie:url (?urn), '%s')", tracker_uri);
+    }
+
+  path = g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+  desktop_uri = photos_utils_convert_path_to_uri (path);
+
+  path = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
+  download_uri = photos_utils_convert_path_to_uri (path);
+
+  path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+  pictures_uri = photos_utils_convert_path_to_uri (path);
+
+  export_path = g_build_filename (path, PHOTOS_EXPORT_SUBPATH, NULL);
+  export_uri = photos_utils_convert_path_to_uri (export_path);
+
+  filter = g_strdup_printf ("(((fn:contains (nie:url (?urn), '%s')"
+                            "   || fn:contains (nie:url (?urn), '%s')"
+                            "   || fn:contains (nie:url (?urn), '%s')"
+                            "   %s)"
+                            "  && !fn:contains (nie:url (?urn), '%s'))"
+                            " || fn:starts-with (nao:identifier (?urn), '%s')"
+                            " || (?urn = nfo:image-category-screenshot))",
+                            desktop_uri,
+                            download_uri,
+                            pictures_uri,
+                            tracker_filter->str,
+                            export_uri,
+                            PHOTOS_QUERY_LOCAL_COLLECTIONS_IDENTIFIER);
+
+  return filter;
+}
+
+
 static gchar *
 photos_source_build_filter_resource (PhotosSource *self)
 {
-  gchar *filter;
+  g_autofree gchar *filter = NULL;
+  gchar *ret_val = NULL;
 
-  if (!self->builtin)
-    filter = g_strdup_printf ("(nie:dataSource (?urn) = '%s')", self->id);
+  g_return_val_if_fail (!self->builtin, NULL);
+
+  if (self->object != NULL)
+    {
+      filter = g_strdup_printf ("(nie:dataSource (?urn) = '%s')", self->id);
+    }
+  else if (self->mount != NULL)
+    {
+      g_autoptr (GFile) root = NULL;
+      g_autofree gchar *uri = NULL;
+
+      root = g_mount_get_root (self->mount);
+      uri = g_file_get_uri (root);
+      filter = g_strdup_printf ("(fn:starts-with (nie:url (?urn), '%s'))", uri);
+    }
   else
-    filter = g_strdup ("(false)");
+    {
+      g_return_val_if_reached (NULL);
+    }
 
-  return filter;
+  ret_val = g_steal_pointer (&filter);
+  return ret_val;
 }
 
 
@@ -91,7 +173,7 @@ photos_source_get_filter (PhotosFilterable *iface)
   g_assert_cmpstr (self->id, !=, PHOTOS_SOURCE_STOCK_ALL);
 
   if (g_strcmp0 (self->id, PHOTOS_SOURCE_STOCK_LOCAL) == 0)
-    return photos_query_builder_filter_local ();
+    return photos_source_build_filter_local ();
 
   return photos_source_build_filter_resource (self);
 }
@@ -105,13 +187,26 @@ photos_source_get_id (PhotosFilterable *filterable)
 }
 
 
+static gboolean
+photos_source_is_search_criterion (PhotosFilterable *filterable)
+{
+  PhotosSource *self = PHOTOS_SOURCE (filterable);
+  gboolean ret_val;
+
+  ret_val = self->mount == NULL;
+  return ret_val;
+}
+
+
 static void
 photos_source_dispose (GObject *object)
 {
   PhotosSource *self = PHOTOS_SOURCE (object);
 
   g_clear_object (&self->icon);
+  g_clear_object (&self->mount);
   g_clear_object (&self->object);
+  g_clear_object (&self->symbolic_icon);
 
   G_OBJECT_CLASS (photos_source_parent_class)->dispose (object);
 }
@@ -146,6 +241,10 @@ photos_source_get_property (GObject *object, guint prop_id, GValue *value, GPara
       g_value_set_string (value, self->id);
       break;
 
+    case PROP_MOUNT:
+      g_value_set_object (value, self->mount);
+      break;
+
     case PROP_NAME:
       g_value_set_string (value, self->name);
       break;
@@ -173,12 +272,51 @@ photos_source_set_property (GObject *object, guint prop_id, const GValue *value,
       break;
 
     case PROP_ID:
-      self->id = g_value_dup_string (value);
-      break;
+      {
+        const gchar *id;
+
+        id = g_value_get_string (value);
+        g_return_if_fail (self->id == NULL || id == NULL);
+
+        if (self->id == NULL && id != NULL)
+          self->id = g_strdup (id);
+
+        break;
+      }
+
+    case PROP_MOUNT:
+      {
+        g_autoptr (GFile) root = NULL;
+        const gchar *type_name;
+        g_autofree gchar *uri = NULL;
+
+        self->mount = G_MOUNT (g_value_dup_object (value));
+        if (self->mount == NULL)
+          break;
+
+        type_name = G_OBJECT_TYPE_NAME (self->mount);
+        root = g_mount_get_root (self->mount);
+        uri = g_file_get_uri (root);
+        self->id = g_strdup_printf ("gd:g-mount:%s:%s", type_name, uri);
+
+        self->icon = g_mount_get_icon (self->mount);
+        self->symbolic_icon = g_mount_get_symbolic_icon (self->mount);
+        self->name = g_mount_get_name (self->mount);
+        break;
+      }
 
     case PROP_NAME:
-      self->name = g_value_dup_string (value);
-      break;
+      {
+        const gchar *name;
+
+        name = g_value_get_string (value);
+        g_return_if_fail (self->name == NULL || name == NULL);
+
+        if (self->name == NULL && name != NULL)
+          self->name = g_strdup (name);
+
+        break;
+      }
 
     case PROP_OBJECT:
       {
@@ -244,6 +382,14 @@ photos_source_class_init (PhotosSourceClass *class)
                                                         NULL,
                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 
+  g_object_class_install_property (object_class,
+                                   PROP_MOUNT,
+                                   g_param_spec_object ("mount",
+                                                        "GMount instance",
+                                                        "A mount point representing a removable device (eg., camera)",
+                                                        G_TYPE_MOUNT,
+                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
+
   g_object_class_install_property (object_class,
                                    PROP_NAME,
                                    g_param_spec_string ("name",
@@ -268,6 +414,7 @@ photos_source_filterable_iface_init (PhotosFilterableInterface *iface)
   iface->get_builtin = photos_source_get_builtin;
   iface->get_filter = photos_source_get_filter;
   iface->get_id = photos_source_get_id;
+  iface->is_search_criterion = photos_source_is_search_criterion;
 }
 
 
@@ -286,6 +433,14 @@ photos_source_new_from_goa_object (GoaObject *object)
 }
 
 
+PhotosSource *
+photos_source_new_from_mount (GMount *mount)
+{
+  g_return_val_if_fail (G_IS_MOUNT (mount), NULL);
+  return g_object_new (PHOTOS_TYPE_SOURCE, "mount", mount, NULL);
+}
+
+
 const gchar *
 photos_source_get_name (PhotosSource *self)
 {
@@ -305,3 +460,17 @@ photos_source_get_icon (PhotosSource *self)
 {
   return self->icon;
 }
+
+
+GMount *
+photos_source_get_mount (PhotosSource *self)
+{
+  return self->mount;
+}
+
+
+GIcon *
+photos_source_get_symbolic_icon (PhotosSource *self)
+{
+  return self->symbolic_icon;
+}
