@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -25,7 +23,7 @@
 #ifndef PHOTOS_SOURCE_H
 #define PHOTOS_SOURCE_H
 
-#include <glib-object.h>
+#include <gio/gio.h>
 #include <goa/goa.h>
 
 G_BEGIN_DECLS
@@ -40,12 +38,18 @@ PhotosSource       *photos_source_new                    (const gchar *id, const
 
 PhotosSource       *photos_source_new_from_goa_object    (GoaObject *object);
 
+PhotosSource       *photos_source_new_from_mount         (GMount *mount);
+
 const gchar        *photos_source_get_name               (PhotosSource *self);
 
 GoaObject          *photos_source_get_goa_object         (PhotosSource *self);
 
 GIcon              *photos_source_get_icon               (PhotosSource *self);
 
+GMount             *photos_source_get_mount              (PhotosSource *self);
+
+GIcon              *photos_source_get_symbolic_icon      (PhotosSource *self);
+
 G_END_DECLS
 
 #endif /* PHOTOS_SOURCE_H */
