@@ -3,10 +3,10 @@
  * Copyright © 2015 – 2017 Red Hat, Inc.
  * Copyright © 2017 Umang Jain
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -29,7 +27,6 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "photos-icons.h"
 #include "photos-tool.h"
 #include "photos-tool-colors.h"
 #include "photos-utils.h"
@@ -71,7 +68,7 @@ static const gdouble BLACKS_STEP = 0.01;
 static const gdouble CONTRAST_DEFAULT = 0.0;
 static const gdouble CONTRAST_MAXIMUM = 1.0;
 static const gdouble CONTRAST_MINIMUM = -1.0;
-static const gdouble CONTRAST_STEP = 0.1;
+static const gdouble CONTRAST_STEP = 0.05;
 static const gdouble EXPOSURE_DEFAULT = 0.0;
 static const gdouble EXPOSURE_MAXIMUM = 3.0;
 static const gdouble EXPOSURE_MINIMUM = -3.0;
@@ -256,7 +253,7 @@ photos_tool_colors_activate (PhotosTool *tool, PhotosBaseItem *item, PhotosImage
     saturation = SATURATION_DEFAULT;
 
   if (photos_base_item_operation_get (item,
-                                      "photos:shadows-highlights",
+                                      "gegl:shadows-highlights",
                                       "highlights", &highlights_real,
                                       "shadows", &shadows_real,
                                       NULL))
@@ -494,7 +491,7 @@ photos_tool_colors_class_init (PhotosToolColorsClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PhotosToolClass *tool_class = PHOTOS_TOOL_CLASS (class);
 
-  tool_class->icon_name = PHOTOS_ICON_IMAGE_ADJUST_COLOR_SYMBOLIC;
+  tool_class->icon_name = "image-adjust-color-symbolic";
   tool_class->name = _("Colors");
 
   object_class->dispose = photos_tool_colors_dispose;
