@@ -4,10 +4,10 @@
  * Copyright © 2015 – 2017 Umang Jain
  * Copyright © 2011 – 2015 Yorba Foundation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -36,7 +34,6 @@
 #include <gtk/gtk.h>
 
 #include "egg-animation.h"
-#include "photos-icons.h"
 #include "photos-image-view.h"
 #include "photos-tool.h"
 #include "photos-tool-crop.h"
@@ -245,7 +242,7 @@ static void
 photos_tool_crop_redraw_damaged_area (PhotosToolCrop *self)
 {
   cairo_rectangle_int_t area;
-  cairo_region_t *region;
+  cairo_region_t *region; /* TODO: use g_autoptr */
   gdouble damage_offset = HANDLE_OFFSET + HANDLE_RADIUS;
   gdouble x;
   gdouble y;
@@ -285,7 +282,7 @@ photos_tool_crop_surface_create (PhotosToolCrop *self)
 static void
 photos_tool_crop_surface_draw (PhotosToolCrop *self)
 {
-  cairo_t *cr;
+  cairo_t *cr; /* TODO: use g_autoptr */
 
   cr = cairo_create (self->surface);
   cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
@@ -725,7 +722,7 @@ photos_tool_crop_set_crop (PhotosToolCrop *self, gdouble event_x, gdouble event_
 static void
 photos_tool_crop_set_cursor (PhotosToolCrop *self)
 {
-  GdkCursor *cursor = NULL;
+  g_autoptr (GdkCursor) cursor = NULL;
   GdkCursorType cursor_type;
   GdkDisplay *display;
   GdkWindow *window;
@@ -783,7 +780,6 @@ photos_tool_crop_set_cursor (PhotosToolCrop *self)
 
  set_cursor:
   gdk_window_set_cursor (window, cursor);
-  g_clear_object (&cursor);
 }
 
 
@@ -918,7 +914,7 @@ static void
 photos_tool_crop_start_constraint_animation (PhotosToolCrop *self)
 {
   GdkFrameClock *frame_clock;
-  PhotosToolCropHelper *helper;
+  g_autoptr (PhotosToolCropHelper) helper = NULL;
 
   g_return_if_fail (self->constraint_animation == NULL);
   g_return_if_fail (self->crop_height >= 0.0);
@@ -951,8 +947,6 @@ photos_tool_crop_start_constraint_animation (PhotosToolCrop *self)
                                                         "y", self->crop_y,
                                                         NULL);
   g_object_add_weak_pointer (G_OBJECT (self->constraint_animation), (gpointer *) &self->constraint_animation);
-
-  g_object_unref (helper);
 }
 
 
@@ -1103,20 +1097,21 @@ static void
 photos_tool_crop_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   PhotosToolCrop *self;
-  GError *error = NULL;
   GtkWidget *orientation_button;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
   gdouble zoom;
   guint active;
 
-  photos_base_item_operation_remove_finish (item, res, &error);
-  if (error != NULL)
-    {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        g_warning ("Unable to process item: %s", error->message);
-      g_error_free (error);
-      return;
-    }
+  {
+    g_autoptr (GError) error = NULL;
+
+    if (!photos_base_item_operation_remove_finish (item, res, &error))
+      {
+        if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+          g_warning ("Unable to process item: %s", error->message);
+        return;
+      }
+  }
 
   self = PHOTOS_TOOL_CROP (user_data);
 
@@ -1446,7 +1441,7 @@ static void
 photos_tool_crop_init (PhotosToolCrop *self)
 {
   GApplication *app;
-  GtkSizeGroup *orientation_size_group = NULL;
+  g_autoptr (GtkSizeGroup) orientation_size_group = NULL;
   GtkStyleContext *context;
   GtkWidget *orientation_box;
   GtkWidget *ratio_grid;
@@ -1507,7 +1502,7 @@ photos_tool_crop_init (PhotosToolCrop *self)
       label = gtk_label_new (_(self->constraints[i].name));
       gtk_container_add (GTK_CONTAINER (grid), label);
 
-      image = gtk_image_new_from_icon_name (PHOTOS_ICON_OBJECT_SELECT_SYMBOLIC, GTK_ICON_SIZE_INVALID);
+      image = gtk_image_new_from_icon_name ("object-select-symbolic", GTK_ICON_SIZE_INVALID);
       gtk_widget_set_no_show_all (image, TRUE);
       gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
       gtk_container_add (GTK_CONTAINER (grid), image);
@@ -1561,8 +1556,6 @@ photos_tool_crop_init (PhotosToolCrop *self)
 
   self->event_x_last = -1.0;
   self->event_y_last = -1.0;
-
-  g_object_unref (orientation_size_group);
 }
 
 
@@ -1572,7 +1565,7 @@ photos_tool_crop_class_init (PhotosToolCropClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PhotosToolClass *tool_class = PHOTOS_TOOL_CLASS (class);
 
-  tool_class->icon_name = PHOTOS_ICON_IMAGE_CROP_SYMBOLIC;
+  tool_class->icon_name = "image-crop-symbolic";
   tool_class->name = _("Crop");
 
   object_class->dispose = photos_tool_crop_dispose;
