@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -54,6 +52,7 @@ struct _PhotosTrackerControllerPrivate
   PhotosQuery *current_query;
   PhotosTrackerQueue *queue;
   PhotosWindowMode mode;
+  gboolean delay_start;
   gboolean is_frozen;
   gboolean is_started;
   gboolean query_queued;
@@ -67,6 +66,7 @@ struct _PhotosTrackerControllerPrivate
 enum
 {
   PROP_0,
+  PROP_DELAY_START,
   PROP_MODE
 };
 
@@ -233,7 +233,11 @@ photos_tracker_controller_cursor_next (GObject *source_object, GAsyncResult *res
   now = g_get_monotonic_time ();
   photos_debug (PHOTOS_DEBUG_TRACKER, "Query Cursor: %" G_GINT64_FORMAT, (now - priv->last_query_time) / 1000000);
 
-  photos_item_manager_add_item_for_mode (PHOTOS_ITEM_MANAGER (priv->item_mngr), priv->mode, cursor);
+  photos_item_manager_add_item_for_mode (PHOTOS_ITEM_MANAGER (priv->item_mngr),
+                                         PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->base_item_type,
+                                         priv->mode,
+                                         cursor);
+
   tracker_sparql_cursor_next_async (cursor,
                                     priv->cancellable,
                                     photos_tracker_controller_cursor_next,
@@ -382,18 +386,22 @@ static void
 photos_tracker_controller_refresh_for_source (PhotosTrackerController *self)
 {
   PhotosTrackerControllerPrivate *priv;
-  PhotosSource *source;
 
   priv = photos_tracker_controller_get_instance_private (self);
 
-  source = photos_query_get_source (priv->current_query);
-  if (source != NULL)
+  if (priv->current_query != NULL)
     {
-      const gchar *id;
+      PhotosSource *source;
 
-      id = photos_filterable_get_id (PHOTOS_FILTERABLE (source));
-      if (g_strcmp0 (id, PHOTOS_SOURCE_STOCK_ALL) == 0)
-        photos_tracker_controller_refresh_internal (self, PHOTOS_TRACKER_REFRESH_FLAGS_NONE);
+      source = photos_query_get_source (priv->current_query);
+      if (source != NULL)
+        {
+          const gchar *id;
+
+          id = photos_filterable_get_id (PHOTOS_FILTERABLE (source));
+          if (g_strcmp0 (id, PHOTOS_SOURCE_STOCK_ALL) == 0)
+            photos_tracker_controller_refresh_internal (self, PHOTOS_TRACKER_REFRESH_FLAGS_NONE);
+        }
     }
 
   priv->refresh_pending = FALSE;
@@ -401,7 +409,7 @@ photos_tracker_controller_refresh_for_source (PhotosTrackerController *self)
 
 
 static void
-photos_tracker_controller_source_object_added_removed (PhotosTrackerController *self)
+photos_tracker_controller_source_object_added_removed (PhotosTrackerController *self, GObject *source)
 {
   PhotosTrackerControllerPrivate *priv;
   PhotosWindowMode mode;
@@ -410,11 +418,17 @@ photos_tracker_controller_source_object_added_removed (PhotosTrackerController *
 
   g_return_if_fail (priv->mode_cntrlr != NULL);
 
+  if (!photos_filterable_is_search_criterion (PHOTOS_FILTERABLE (source)))
+    goto out;
+
   mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
   if (mode == priv->mode)
     photos_tracker_controller_refresh_for_source (self);
   else
     priv->refresh_pending = TRUE;
+
+ out:
+  return;
 }
 
 
@@ -518,6 +532,10 @@ photos_tracker_controller_set_property (GObject *object, guint prop_id, const GV
 
   switch (prop_id)
     {
+    case PROP_DELAY_START:
+      priv->delay_start = g_value_get_boolean (value);
+      break;
+
     case PROP_MODE:
       priv->mode = (PhotosWindowMode) g_value_get_enum (value);
       break;
@@ -574,11 +592,21 @@ photos_tracker_controller_class_init (PhotosTrackerControllerClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  class->base_item_type = G_TYPE_NONE;
+
   object_class->constructed = photos_tracker_controller_constructed;
   object_class->dispose = photos_tracker_controller_dispose;
   object_class->set_property = photos_tracker_controller_set_property;
   object_class->finalize = photos_tracker_controller_finalize;
 
+  g_object_class_install_property (object_class,
+                                   PROP_DELAY_START,
+                                   g_param_spec_boolean ("delay-start",
+                                                         "Delay start",
+                                                         "Don't start issuing queries immediately on startup",
+                                                         FALSE,
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+
   g_object_class_install_property (object_class,
                                    PROP_MODE,
                                    g_param_spec_enum ("mode",
@@ -639,6 +667,9 @@ photos_tracker_controller_start (PhotosTrackerController *self)
 
   priv = photos_tracker_controller_get_instance_private (self);
 
+  if (priv->delay_start)
+    return;
+
   if (priv->is_started)
     return;
 
