@@ -2,10 +2,10 @@
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2014 – 2017 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Based on code from:
@@ -28,11 +26,14 @@
 #include <gio/gio.h>
 
 #include "photos-base-manager.h"
+#include "photos-filterable.h"
 #include "photos-item-manager.h"
 #include "photos-offset-search-controller.h"
 #include "photos-query-builder.h"
 #include "photos-search-context.h"
 #include "photos-search-controller.h"
+#include "photos-search-type.h"
+#include "photos-source.h"
 #include "photos-tracker-search-controller.h"
 #include "photos-utils.h"
 
@@ -80,6 +81,38 @@ photos_tracker_search_controller_get_query (PhotosTrackerController *trk_cntrlr)
 }
 
 
+static void
+photos_tracker_search_controller_search_changed (PhotosTrackerSearchController *self)
+{
+  GObject *object;
+  const gchar *search_type_id;
+  const gchar *source_id;
+  const gchar *str;
+
+  object = photos_base_manager_get_active_object (self->src_mngr);
+  source_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+
+  object = photos_base_manager_get_active_object (self->srch_typ_mngr);
+  search_type_id = photos_filterable_get_id (PHOTOS_FILTERABLE (object));
+
+  str = photos_search_controller_get_string (self->srch_cntrlr);
+
+  if (g_strcmp0 (search_type_id, PHOTOS_SEARCH_TYPE_STOCK_ALL) != 0
+      || g_strcmp0 (source_id, PHOTOS_SOURCE_STOCK_ALL) != 0
+      || (str != NULL && str [0] != '\0'))
+    {
+      photos_tracker_controller_refresh_for_object (PHOTOS_TRACKER_CONTROLLER (self));
+    }
+}
+
+
+static void
+photos_tracker_search_controller_source_active_changed (PhotosTrackerSearchController *self)
+{
+  photos_tracker_search_controller_search_changed (self);
+}
+
+
 static void
 photos_tracker_search_controller_search_match_active_changed (PhotosTrackerSearchController *self)
 {
@@ -93,6 +126,20 @@ photos_tracker_search_controller_search_match_active_changed (PhotosTrackerSearc
 }
 
 
+static void
+photos_tracker_search_controller_search_string_changed (PhotosTrackerSearchController *self)
+{
+  photos_tracker_search_controller_search_changed (self);
+}
+
+
+static void
+photos_tracker_search_controller_search_type_active_changed (PhotosTrackerSearchController *self)
+{
+  photos_tracker_search_controller_search_changed (self);
+}
+
+
 static GObject *
 photos_tracker_search_controller_constructor (GType type,
                                               guint n_construct_params,
@@ -138,30 +185,34 @@ photos_tracker_search_controller_init (PhotosTrackerSearchController *self)
   state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
 
   self->src_mngr = g_object_ref (state->src_mngr);
-  g_signal_connect_swapped (self->src_mngr,
-                            "active-changed",
-                            G_CALLBACK (photos_tracker_controller_refresh_for_object),
-                            self);
+  g_signal_connect_object (self->src_mngr,
+                           "active-changed",
+                           G_CALLBACK (photos_tracker_search_controller_source_active_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   self->srch_mtch_mngr = g_object_ref (state->srch_mtch_mngr);
-  g_signal_connect_swapped (self->srch_mtch_mngr,
-                            "active-changed",
-                            G_CALLBACK (photos_tracker_search_controller_search_match_active_changed),
-                            self);
+  g_signal_connect_object (self->srch_mtch_mngr,
+                           "active-changed",
+                           G_CALLBACK (photos_tracker_search_controller_search_match_active_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   self->srch_typ_mngr = g_object_ref (state->srch_typ_mngr);
-  g_signal_connect_swapped (self->srch_typ_mngr,
-                            "active-changed",
-                            G_CALLBACK (photos_tracker_controller_refresh_for_object),
-                            self);
+  g_signal_connect_object (self->srch_typ_mngr,
+                           "active-changed",
+                           G_CALLBACK (photos_tracker_search_controller_search_type_active_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   self->offset_cntrlr = photos_offset_search_controller_dup_singleton ();
 
   self->srch_cntrlr = g_object_ref (state->srch_cntrlr);
-  g_signal_connect_swapped (self->srch_cntrlr,
-                            "search-string-changed",
-                            G_CALLBACK (photos_tracker_controller_refresh_for_object),
-                            self);
+  g_signal_connect_object (self->srch_cntrlr,
+                           "search-string-changed",
+                           G_CALLBACK (photos_tracker_search_controller_search_string_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 }
 
 
@@ -181,5 +232,8 @@ photos_tracker_search_controller_class_init (PhotosTrackerSearchControllerClass
 PhotosTrackerController *
 photos_tracker_search_controller_dup_singleton (void)
 {
-  return g_object_new (PHOTOS_TYPE_TRACKER_SEARCH_CONTROLLER, "mode", PHOTOS_WINDOW_MODE_SEARCH, NULL);
+  return g_object_new (PHOTOS_TYPE_TRACKER_SEARCH_CONTROLLER,
+                       "delay-start", TRUE,
+                       "mode", PHOTOS_WINDOW_MODE_SEARCH,
+                       NULL);
 }
